<?php


namespace App\Helpers;


use GuzzleHttp\Client;
use GuzzleHttp\Exception\RequestException;
use GuzzleHttp\RequestOptions;
class ApiService
{
    protected $client;
    protected $baseUrl;
    protected $token;

    public function __construct()
    {
        $this->baseUrl = config('api.base_url'); // Fetch base URL from config
        $this->token = config('api.x_auth_token'); // Fetch X-Auth-Token from config
        $this->client = new Client([
            'base_uri' => $this->baseUrl,
            'headers' => [
                'x-auth-token' => $this->token,
                'Content-Type' => 'application/json',
                // Add other headers if needed
            ],
            'curl' => [
               CURLOPT_SSL_VERIFYPEER => false, // Disable verification of SSL certificate
               CURLOPT_SSL_VERIFYHOST => false, // Disable verification of host name in SSL certificate
            ],
	    RequestOptions::VERIFY => false,
        ]);
    }

    public function callAPI($method, $endpoint, $data = [])
    {
        $options = [
            'headers' => [
                'x-auth-token' => $this->token,
                'Content-Type' => 'application/json',
                // Add other headers if needed
            ],
        ];

        if (!empty($data)) {
            $options['json'] = $data;
        }

        try {
            $response = $this->client->request($method, $endpoint, $options);
            return json_decode($response->getBody()->getContents(), true);
        }catch (RequestException $e){
            if ($e->hasResponse()) {
                // Get the response body
                $responseBody = $e->getResponse()->getBody()->getContents();
                // Return response as array
                return json_decode($responseBody, true);
            } else {
                // Return a generic error message if the response is not available
                return ['error' => 'An error occurred while processing your request.'];
            }
        }


    }
}
