<?php

namespace App\Http\Controllers;

use App\Models\EmployeeModel;
use App\Models\User;
use App\Models\Currency;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\Validator;
use App\Http\Requests\ResetPasswordRequest;
use Illuminate\Support\Facades\Password;
use Mail;


class AuthController extends Controller
{

    protected function get_user($id){

        $user= User::join('currencies','users.currency_id','currencies.cu_id')->where('id','=',$id)->first();

        $response = [
            'status' =>"success",
            'user' =>$user
        ];

        return response($response, 200);

    }


    protected function update_profile(Request $request,$id){


        $request->validate([
            'name' => ['required', 'string', 'max:255'],
            'l_name' => ['required', 'string', 'max:255'],
            'email' => ['required', 'string', 'email', 'max:255', 'unique:users'],
            'phone_number' => ['required'],
            'currency_id' => ['required'],
        ]);

        $user= User::where('id','=',$id)->first();

        $user->name = $request->name;
        $user->l_name = $request->l_name;
        $user->email = $request->email;
        $user->phone_number = $request->phone_number;
        $user->dob = $request->dob;
        $user->address = $request->address;
        $user->currency_id = $request->currency_id;
        $user->save();

        $response = [
            'status' =>"success",
            'user' =>$user
        ];

        return response($response, 200);

    }


    //  ======================== FUNCTIONS =============== //
    public function decode_image($img , $unique_id, $path_url, $prefix, $random, $postfix){

        $data = base64_decode($img);
        $file_name = $prefix.$unique_id.$random.$postfix.'.jpeg';
        $file = $path_url.$file_name;
//        $success = file_put_contents($file, $data);
//        $file->move('uploads/employee/',$data);
        Storage::disk('public')->put($file_name, $data);

        return $file_name;

    }


    protected function upload_img(Request $request,$id){

        $profile_picture = $request->image;
        $postfix = 'morlaApp';
        $upd_prefix = date('d-m-y');
        $random = rand(1,10000);
        $image_path_url= "https://eigix.net/morlaApp/public/uploads/";
        $uploaded_image = $this->decode_image($profile_picture, $id, $image_path_url, $upd_prefix, $random, $postfix);
        $query = DB::table('users')->where('id', $id)->update(array('profile_picture' => $uploaded_image));


        $response = [
            'status' =>"sucess",
            'data' =>$uploaded_image
        ];

        return response($response, 200);

    }


    protected function email_validation(Request $request){

        $fields = $request->validate([
            'name' => ['required', 'string', 'max:255'],
            'email' => ['required', 'string', 'email', 'max:255', 'unique:users'],
            'password' => ['required', 'string', 'min:8'],
        ]);

        $response = [
            'status' =>"sucess",
            'message' =>"Email Verified"
        ];

        return response($response, 200);

    }



    protected function register(Request $request)
    {

        $fields = $request->validate([
            'name' => ['required', 'string', 'max:255'],
            'email' => ['required', 'string', 'email', 'max:255', 'unique:users'],
            'password' => ['required', 'string', 'min:8'],
            'phone_number' => ['required'],
            'currency_id' => ['required'],
        ]);


        $user = User::create([
            'name' => $fields['name'],
            'email' => $fields['email'],
            'password' => bcrypt($fields['password']),
            'phone_number' => $fields['phone_number'],
            'currency_id' => $fields['currency_id'],
        ]);


        $token = $user->createToken('myapptoken')->plainTextToken;

        $response = [
            'user' =>$user,
            'token' =>$token
        ];


        return response($response, 201);

    }

    protected function login(Request $request)
    {

        $fields = $request->validate([
            'email' => ['required', 'string', 'email', 'max:255'],
            'password' => ['required', 'string', 'min:8'],
        ]);


        $user = User::where('email' , $fields['email'])->first();

        if(!$user || !Hash::check($fields['password'], $user->password)){
            return response([
                'message' => "Wrong Credentials"
            ], 401);
        }


        $token = $user->createToken('myapptoken')->plainTextToken;

        $response = [
            'user' =>$user,
            'token' =>$token
        ];


        return response($response, 201);

    }

    public function logout(Request $request){

        $request->user()->currentAccessToken()->delete();

        return [
            'message' => 'Logged Out'
        ];
    }


    /*** SIGNUP with Google/Apple  ***/
    public  function signup_with_social(Request $req){
        $response = array();


        if (isset($req->accountType) && isset($req->oneSignalId)) {

            switch($req->accountType){
                case 'google':
                    $accesstoken_exists = User::where('google_access_token', $req->googleAccessToken)->whereNotNull('google_access_token')->get()->count();
                    $useremail_exists = User::where('email', $req->userEmail)->get()->count();


                    if($useremail_exists == 1){

                        //returning old data
                        if($accesstoken_exists == 0){
                            //updating user info with google access token and giving old data
                            $query = User::where('email',$req->userEmail)->update(array('account_type'=> 'Both','google_access_token' => $req->googleAccessToken));
                            if($query){
                                $user_details = User::where('google_access_token', $req->googleAccessToken)->first();
                                if($user_details){
                                    if(password_verify('123456', $user_details->password)){
                                        $response["temp_password_flag"] = "1";
                                    } else{
                                        $response["temp_password_flag"] = "0";
                                    }
                                    if($user_details->profile_picture != ""){
                                        $base_url_profile_images = config('base_urls.profile_images_base_url');
                                        $user_details->profile_picture = $base_url_profile_images.$user_details->profile_picture;
                                    }
                                    //updating one signal device id for user
                                    User::where('id', $user_details->id)->update(array('one_signal_id' => $req->oneSignalId));
                                    $response["code"] = 200;
                                    $response["status"] = "success / Signed in with Google";
                                    $response["data"] = $user_details;
                                }
                            }

                        } else{
                            //simple returning old data
                            $user_details = User::where('google_access_token', $req->googleAccessToken)->first();
                            if($user_details){
                                if(password_verify('123456', $user_details->password)){
                                    $response["temp_password_flag"] = "1";
                                } else{
                                    $response["temp_password_flag"] = "0";
                                }
                                if($user_details->profile_picture != ""){
                                    $base_url_profile_images = config('base_urls.profile_images_base_url');
                                    $user_details->profile_picture = $base_url_profile_images.$user_details->profile_picture;
                                }
                                //updating one signal device id for user
                                User::where('id', $user_details->id)->update(array('one_signal_id' => $req->oneSignalId));
                                $response["code"] = 200;
                                $response["status"] = "success / Signed in with Google";
                                $response["data"] = $user_details;
                            }

                        }

                    } else{
                        //new google register
                        $user_data = array(
                            'name' => (isset($req->userName) ? $req->userName: ''),
                            'email' => (isset($req->userEmail) ? $req->userEmail: ''),
                            'created_at' => date('Y-m-d H:i:s'),
                            'password' => password_hash('123456', PASSWORD_DEFAULT),
                            'roles_id' => '1',
                            'gender' => 'Male',
                            'status' => 'Active',
                            'one_signal_id' => $req->oneSignalId,
                            'account_type' => 'SignupWithSocial',
                            'social_acc_type'=> 'Google',
                            'verify_code' => '',
                            'profile_picture' => '',
                            'google_access_token' => $req->googleAccessToken,

                        );

                        $query =  User::insert($user_data);
                        if($query){
                            $user_details = User::where('google_access_token', $req->googleAccessToken)->first();
                            if (!empty($user_details)) {
                                if($user_details->profile_picture != ""){
                                    $base_url_profile_images = config('base_urls.profile_images_base_url');
                                    $user_details->profile_picture = $base_url_profile_images.$user_details->profile_picture;
                                }

                                if($user_details->profile_picture != ""){
                                    $base_url_profile_images = config('base_urls.profile_images_base_url');
                                    $user_details->profile_picture = $base_url_profile_images.$user_details->profile_picture;
                                }
                                $response["code"] = 200;
                                $response["status"] = "success / Signed in with Google";
                                $response["temp_password_flag"] = "1";
                                $response["data"] = $user_details;
                            } else{
                                $response["code"] = 404;
                                $response["temp_password_flag"] = "0";
                                $response["status"] = "error";
                                $response["message"] = "User not found";
                            }
                        } else{
                            $response["code"] = 500;
                            $response["status"] = "error";
                            $response["temp_password_flag"] = "0";
                            $response["message"] = "Something went wrong while inserting data.";
                        }

                    }

                    break;

                case 'apple':
                    $appleid_exists = User::where('apple_id', $req->appleId)->get()->count();
                    if(isset($req->userEmail)){
                        $useremail_exists = User::where('email', $req->userEmail)->get()->count();
                    } else{
                        $useremail_exists = "0";
                    }

                    if($useremail_exists == 1){
                        //returning old data
                        if($appleid_exists == 0){
                            //updating user info with Apple id and giving old data
                            $query = User::where('email',  $req->userEmail)->update(array('account_type'=> 'Both','apple_id' => $req->appleId));
                            if($query){
                                $user_details = User::where('apple_id', $req->appleId)->first();
                                if($user_details){
                                    $response["code"] = 200;
                                    if(password_verify('123456', $user_details->password)){
                                        $response["temp_password_flag"] = "1";
                                    } else{
                                        $response["temp_password_flag"] = "0";
                                    }
                                    if($user_details->profile_picture != ""){
                                        $base_url_profile_images = config('base_urls.profile_images_base_url');
                                        $user_details->profile_picture = $base_url_profile_images.$user_details->profile_picture;
                                    }
                                    //updating one signal device id for user
                                    User::where('id', $user_details->id)->update(array('one_signal_id' => $req->oneSignalId));
                                    $response["status"] = "success / Signed in with Apple";
                                    $response["data"] = $user_details;
                                }
                            }
                        } else{
                            //simple returning old data
                            $user_details = User::where('apple_id', $req->appleId)->first();
                            if($user_details){
                                $response["code"] = 200;
                                if(password_verify('123456', $user_details->password)){
                                    $response["temp_password_flag"] = "1";
                                } else{
                                    $response["temp_password_flag"] = "0";
                                }
                                if($user_details->profile_picture != ""){
                                    $base_url_profile_images = config('base_urls.profile_images_base_url');
                                    $user_details->profile_picture = $base_url_profile_images.$user_details->profile_picture;
                                }
                                //updating one signal device id for user
                                User::where('id', $user_details->id)->update(array('one_signal_id'=> $req->oneSignalId));
                                $response["status"] = "success / Signed in with Apple";
                                $response["data"] = $user_details;
                            }
                        }
                    }  else {
                        if($appleid_exists == 1){
                            //updating user info with Apple id and giving old data
                            $user_details = User::where('apple_id', $req->appleId)->first();
                            if($user_details){
                                $response["code"] = 200;
                                if(password_verify('123456', $user_details->password)){
                                    $response["temp_password_flag"] = "1";
                                } else{
                                    $response["temp_password_flag"] = "0";
                                }
                                if($user_details->profile_picture != ""){
                                    $base_url_profile_images = config('base_urls.profile_images_base_url');
                                    $user_details->profile_picture = $base_url_profile_images.$user_details->profile_picture;
                                }

                                //updating one signal device id for user
                                User::where('id', $user_details->id)->update(array('one_signal_id' => $req->oneSignalId));
                                $response["status"] = "success / Signed in with Apple";
                                $response["data"] = $user_details;
                            }

                        } else{
                            //new Apple register
                            $user_data = array(
                                'name' => (isset($req->userName) ? $req->userName: ''),
                                'email' => (isset($req->userEmail) ? $req->userEmail: ''),
                                'created_at' => date('Y-m-d H:i:s'),
                                'password' => password_hash('123456', PASSWORD_DEFAULT),
                                'roles_id' => '1',
                                'gender' => 'Male',
                                'status' => 'Active',
                                'one_signal_id' => $req->oneSignalId,
                                'account_type' => 'SignupWithSocial',
                                'social_acc_type'=> 'Apple',
                                'verify_code' => '',
                                'profile_picture' => '',
                                'apple_id' => $req->appleId
                            );
                            $query =  User::insert($user_data);
                            if($query){
                                $user_details = User::where('apple_id', $req->appleId)->first();
                                if (!empty($user_details)) {
                                    if($user_details->profile_picture != ""){
                                        $base_url_profile_images = config('base_urls.profile_images_base_url');
                                        $user_details->profile_picture = $base_url_profile_images.$user_details->profile_picture;
                                    }

                                    $response["code"] = 200;
                                    $response["temp_password_flag"] = "1";
                                    $response["status"] = "success / Signed in with Apple";
                                    $response["data"] = $user_details;
                                } else{
                                    $response["code"] = 404;
                                    $response["status"] = "error";
                                    $response["temp_password_flag"] = "0";
                                    $response["message"] = "User not found";
                                }
                            } else{
                                $response["code"] = 500;
                                $response["status"] = "error";
                                $response["temp_password_flag"] = "0";
                                $response["message"] = "Something went wrong while inserting data.";
                            }
                        }
                    }
                    break;

                default:
                    $response["code"] = 500;
                    $response["status"] = "error";
                    $response["temp_password_flag"] = "0";
                    $response["message"] = "Invalid Account Type.";
            }

        } else {
            $response["code"] = 404;
            $response["status"] = "error";
            $response["temp_password_flag"] = "0";
            $response["message"] = "All fields are needed.";
        }


        return response()
            ->json(array('temp_password_flag' => $response["temp_password_flag"],'status' => $response["status"], isset($response["message"]) ? 'message' : 'data' => isset($response["message"]) ? $response["message"] : $response["data"]))
            ->header('Content-Type', 'application/json');
    }

    /*** GET USER TOKEN ***/
    public function get_token(Request $req){
        $response["code"] = 200;
        $response["status"] = "success";
        $response["token"] = csrf_token();
        return $response;
    }

    /*** FORGOT PASSWORD USER ***/
    public function forgot_password(Request $req){
//        $postdata=file_get_contents("php://input");
//        $req=json_decode($postdata);
        $response=array();
        $requestType=$req->requestType;

        switch($requestType){
            case 'forgot_password':
                $email	    =   $req->email;
                if(isset($email)){
                    $get_query = User::where('email', $email)->get()->count();
                    if($get_query > 0){
                        $update['verify_code'] = $this->generate_code(4);
                        $query =   User::where('email', '=', $email)->update($update);
                        if($query){
                            $email_template = "Dear,
    						Here is your verification code. This code can only be used for one time only.
    						{verify_code}";
                            $subject = "Verification Code";
                            $message = $email_template;
                            $message = str_replace("{verify_code}", $update['verify_code'], $message);
                            $result = $this->send_email($email, $subject, $message);

                            if($result){
                                $response['code']		= 200;
                                $response['status'] ='Success';
                                $response['message']='Verification Code is Successfully sent to your Email Address';
                            } else {
                                $response['code']		= 404;
                                $response['status'] ='error';
                                $response['message']='Verification Code is not sent to your Email Address';
                            }
                        } else {
                            $response['code']		= 404;
                            $response['status'] = "Error";
                            $response['msg']    = "Code is not generated";
                        }
                    } else {
                        $response['code']		= 404;
                        $response['status']="error";
                        $response['message']="email not found";
                    }
                } else{
                    $response['code']		= 404;
                    $response['status'] ='error';
                    $response['message']='email is required';
                }

                return response()
                    ->json(array('status' => $response["status"], isset($response["message"]) ? 'message' : 'data' => isset($response["message"]) ? $response["message"] : $response["data"]))
                    ->header('Content-Type', 'application/json');
                break;

            case 'match_code':
                $email      = $req->email;
                $code       = $req->code;
                $email_get  = User::where('email', $email)->first();
                if($email_get){
                    $matchCode=$email_get->verify_code;
                    if($matchCode == $code){
                        $response['code']		= 200;
                        $response['status'] = 'Success';
                        $response['message']= 'Your Code is Successfully Verified';
                    } else{
                        $response['code']		= 404;
                        $response['status'] = 'error';
                        $response['message']= 'Code doesnot match,Try Again Please';
                    }
                } else{
                    $response['code']		= 404;
                    $response['status'] = 'error';
                    $response['message']= 'No such Email is present in Database';
                }

                return response()
                    ->json(array('status' => $response["status"], isset($response["message"]) ? 'message' : 'data' => isset($response["message"]) ? $response["message"] : $response["data"]))
                    ->header('Content-Type', 'application/json');
                break;

            case 'reset_password':

                $fields = $req->validate([
                    'password' => ['required', 'string', 'min:8'],
                    'confirmPassword' => ['required', 'string', 'min:8'],
                ]);

                $email	 	    = $req->email;
                $password	    = $req->password;
                $c_password	    = $req->confirmPassword;
                $email_get      = User::where('email', $email)->first();
                $old_password   = $email_get->password;
                if($email_get){
                    if($password == $c_password){
                        if($old_password != $password){
                            $update['password'] = password_hash($password,PASSWORD_BCRYPT);
                            $query =   User::where('email', $email)->update($update);
                            if($query){
                                $response['code']		= 200;
                                $response['status'] = 'Success';
                                $response['message']= 'You have successfully Updated Your Password';
                            } else {
                                $response['code']		= 404;
                                $response['status'] = 'error';
                                $response['message']= 'Password is not Updated';
                            }
                        } else {
                            $response['code']		= 404;
                            $response['status'] = 'error';
                            $response['message']= 'This is your OLD password,Please try with new Strong Password';
                        }
                    } else {
                        $response['code']	= 404;
                        $response['status'] = 'error';
                        $response['message']= 'Password and confirm password are not Matching';
                    }
                } else {
                    $response['code']	= 404;
                    $response['status'] = 'error';
                    $response['message']= 'No such Email is present in Database';
                }

                return response()
                    ->json(array('status' => $response["status"], isset($response["message"]) ? 'message' : 'data' => isset($response["message"]) ? $response["message"] : $response["data"]))
                    ->header('Content-Type', 'application/json');
                break;
        }
    }

    /*** CHECK ALREADY REGISTERED ***/
    public  function check_already_registered(Request $req){
        $response = array();


        if (isset($req->userEmail) && isset($req->userName)){
            $username_exists = User::where('user_name', $req->userName)->get()->count();
            if ($username_exists == 0) {
                $useremail_exists = User::where('email', $req->userEmail)->get()->count();
                if ($useremail_exists == 0) {
                    $response["code"] = 200;
                    $response["status"] = "success";
                    $response["data"] = 'Unique email and username';
                } else {
                    $response["code"] = 404;
                    $response["status"] = "error";
                    $response["message"] = "Email address already exists.";
                }
            } else {
                $response["code"] = 404;
                $response["status"] = "error";
                $response["message"] = "User name already exists.";
            }
        } else {
            $response["code"] = 404;
            $response["status"] = "error";
            $response["message"] = "All fields are needed.";
        }


        return response()
            ->json(array('status' => $response["status"], isset($response["message"]) ? 'message' : 'data' => isset($response["message"]) ? $response["message"] : $response["data"]))
            ->header('Content-Type', 'application/json');

    }


    /*** GENERATE CODE ***/
    public function generate_code($limit){
        $permitted_chars = '0123456789';
        return substr(str_shuffle($permitted_chars), 0, $limit);
    }


    /*** Get Currency ***/
    public function get_currency(){

        $currency = Currency::all();
        return response()
            ->json(array('currency'=>$currency));
    }

//    /*** SEND MAIL ***/
//    public function send_email($to, $subject, $msg) {
//
//        $data = array('msg'=>$msg);
//        $dataSubject = $subject;
//
//        $mail = Mail::send('forget_mail', $data, function($message) {
//            $message->to('nabeel.py@gmail.com', 'Tutorials Point')->subject("Verification Code");
//        });
//
//        return $mail;
//    }

    public function send_email($to, $subject, $msg) {

        $data = array('msg'=>$msg);
        $dataSubject = $subject;

        $details = [
            'title' => 'Mail from Eigix.com',
            'body' => $msg
        ];

        $mail = \Mail::to($to)->send(new \App\Mail\SendMail($details));

        return $mail;
    }


}
