<?php

namespace App\Http\Controllers;

use App\Helpers\ApiService;
use App\Helpers\CustomerLogic;
use App\Models\StripeCustomer;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Cookie;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Response;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Str;
use Stripe\Charge;
use Stripe\Exception\ApiErrorException;
use Stripe\Invoice;
use Stripe\PaymentIntent;
use Stripe\PaymentMethod;
use Stripe\Price;
use Stripe\Product;
use Stripe\Stripe;
use Stripe\Subscription;
use Illuminate\Support\Facades\Http;
use function Symfony\Component\Mime\Header\all;
use Illuminate\Support\Facades\Mail;

class DashboardController extends Controller
{
    protected $apiService;
    protected $referralUrl;
    public function __construct(APIService $apiService)
    {
        $this->apiService = $apiService;
        $this->referralUrl = config('services.referral_api.base_url');
    }

    public function dashboard(Request $request)
    {
        $customer_info = Cookie::get('customer_info');
        $customer_info = json_decode($customer_info, true);
        $response = CustomerLogic::get_balance($this->apiService, $customer_info['id']);
        if ($response['response_code'] != 200){
            if ($response['error'] == 'Account Not Found'){
                return redirect()->route('logout');
            }
            $balance = '';
            notify()->error($response['error'],"Error");

        }
        $balance = $response['data'] ?? '';
        $product_res = CustomerLogic::productlist($this->apiService);
        if ($product_res['response_code'] == 400){
            return redirect()->route('logout');
        }
        $products = $product_res['data'] ?? [];
        $response = $this->apiService->callAPI('POST', 'api/global_api/', [
            "action" => "get_country_list",
            "device_id" =>  "",
        ]);
        $country_list = [];
        if ($response['response_code'] == 200){
            $country_list = $response['data'] ?? [];
	}

	$csc_data = array(
          [
              'username' => '6300512225',
              'cloud_id' => 'APPLE'
          ],
          [
              'username' => '8381624081',
              'cloud_id' => 'APPLE'
          ],
          [
              'username' => '3457060031',
              'cloud_id' => 'GOOGLE'
	  ],
	  [
	      'username' => '8083301342',
	      'cloud_id' => 'GOOGLE'
	  ],
	  [
	      'username' => '466453679',
	      'cloud_id' => 'GOOGLE'
	  ]
        );
        $cloud_id = env('CLOUD_ID');
        foreach ($csc_data as $csc){
            if ($csc['username'] == $customer_info['username']){
                $cloud_id = $csc['cloud_id'];
            }
        }
	//return view('blank', compact('customer_info', 'cloud_id'));
	return view('update.dashboard', compact('customer_info', 'balance', 'products', 'country_list', 'cloud_id'));
    }

    public function add_balance(Request $request)
    {
        $validation = Validator::make($request->all(), [
           'balance' => 'required',
        ]);
        if ($validation->fails()){
            notify()->error('Amount filed is required.',"Error");
            return redirect()->back();
        }
        $customer_info = Cookie::get('customer_info');
        $customer_info = json_decode($customer_info, true);

        $response = CustomerLogic::add_balance($this->apiService, $customer_info, $request->balance);
        if ($response['response_code'] == 400){
            return redirect()->route('logout');
        }

        notify()->success($response['success'],"Success");
        return redirect()->back();
    }

    public function profile()
    {
        $customer_info = Cookie::get('customer_info');
        $customer_info = json_decode($customer_info, true);

        $response = $this->apiService->callAPI('POST', 'api/user_profile/', [
            "id" => $customer_info['id'],
            "token" => Cookie::get('c_auth_token'),
            "action" =>  "read_profile",
            "device_id" =>  "",
        ]);
        if ($response['response_code'] == 400){
            return redirect()->route('logout');
        }
        $profile = $response['data'];
        return view('update.profile', compact('customer_info',  'profile'));
    }

    public function payment_history()
    {
        $customer_info = Cookie::get('customer_info');
        $customer_info = json_decode($customer_info, true);
        $response = $this->apiService->callAPI('POST', 'api/user_charges_history/', [
            "id" => $customer_info['id'],
            "token" => Cookie::get('c_auth_token'),
            "action" =>  "charges_history_list",
            "device_id" =>  "",
            "object_where_params" => [
                "from_date" => "",
                "to_date" => "",
            ],
            "start_limit" => "1",
            "end_limit" => "100000000"
        ]);
        if ($response['response_code'] == 400){
            return redirect()->route('logout');
        }

        $payments = $response['data'] ?? [];
        $data = $payments;
        $groupedData = collect($data)->map(function ($item) {
            // Parse the date and format it
            $item['date'] = Carbon::parse($item['date']);
            return $item;
        })->groupBy(function ($item) {
            return $item['date']->format('Y'); // Group by year
        })->map(function ($yearGroup) {
            return $yearGroup->groupBy(function ($item) {
                return $item['date']->format('m'); // Group by month
            })->map(function ($monthGroup) {
                return $monthGroup->groupBy(function ($item) {
                    return $item['date']->format('d'); // Group by day
                });
            });
        });
        $payments  = $groupedData;
        return view('update.payment_history', compact('payments'));
    }
    public function refer_and_earn(){
	$customer_info = Cookie::get('customer_info');
        $customer_info = json_decode($customer_info, true);
	$response = Http::withOptions([
                'curl' => [
                        CURLOPT_SSL_VERIFYHOST => false,
                        CURLOPT_SSL_VERIFYPEER => false,
                        // Other cURL options if needed
                ],
                ])->get($this->referralUrl . '/get_Promo_code.php', [
                'sipUsername' => $customer_info['number']
                ]);

	$response= (array)json_decode($response);
	$promo_code =$response['Promo Code'];
	return view('referral_and_earn', compact('promo_code'));
    }
    public function my_referrals(){
	$customer_info = Cookie::get('customer_info');
        $customer_info = json_decode($customer_info, true);
	$summary_response = Http::withOptions([
                'curl' => [
                        CURLOPT_SSL_VERIFYHOST => false,
                        CURLOPT_SSL_VERIFYPEER => false,
                        // Other cURL options if needed
                ],
        ])->get($this->referralUrl . '/get_referral_summary.php', [
                'sipUsername' => $customer_info['number']
        ]);
	$summary_response= (array)json_decode($summary_response);
	$detail_response = Http::withOptions([
                'curl' => [
                        CURLOPT_SSL_VERIFYHOST => false,
                        CURLOPT_SSL_VERIFYPEER => false,
                        // Other cURL options if needed
                ],
        ])->get($this->referralUrl . '/get_referrals.php', [
                'sipUsername' => $customer_info['number']
        ]);
	$detail_response= (array)json_decode($detail_response);
	if(isset($detail_response['error'])){
		$detail_response=array();
	}
	return view('my_referral', compact('detail_response',  'summary_response'));
    }
    public function your_subscriptions()
    {
        $customer_info = Cookie::get('customer_info');
        $customer_info = json_decode($customer_info, true);
        $response = $this->apiService->callAPI('POST', 'api/user_my_order/', [
            "id" => $customer_info['id'],
            "token" => Cookie::get('c_auth_token'),
            "action" =>  "list",
            "start_limit" => "1",
            "end_limit" => "1000000",
            "subaction_ios" => "",
            "device_id" =>  "",
            "object_where_params" => [
                "from_date" => "",
                "to_date" => "",
                "order" => "",
                "payment_status" => "",
                "order_status" => "",
                "product_id" => "",
            ],

        ]);

        if ($response['response_code'] == 400){
            return redirect()->route('logout');
        }
        $subscriptions_data = $response['data'] ?? [];
        $subscriptions_data = collect($subscriptions_data)->where('order_status', 'Active');

        $product_res = $this->apiService->callAPI('GET', 'product-list.php', []);

        $subscriptions = [];
        $packages = [];

        foreach ($subscriptions_data as $data){
            foreach ($product_res as $product){
                if ($data['product_id'] == $product['id']){
                    if($product['billing_type'] == 0){
                        //package
                        $packages[] = $data;
                    }else{
                        $subscriptions[] = $data;
                    }
                }
            }
        }

        $stripe_customer = @StripeCustomer::where('customer_id', $customer_info['id'])->first()->stripe_customer_id;
        Stripe::setApiKey(env('STRIPE_SECRET'));

        $final_subscription = array();

        if ($stripe_customer != null){
            try {
                $subscriptions = Subscription::all([
                    'customer' => $stripe_customer,
                    'status' => 'active'
                ]);

                foreach ($subscriptions->data as $subscription) {

                    $meta_data = $subscription->metadata;

                    $product_name = @collect($product_res)->where('id', $meta_data->product_id)->first()['name'];

                    $expirationDate = $subscription->current_period_end;

                    // Convert the timestamp to a human-readable date
                    $expirationDateFormatted = date('Y-m-d H:i:s', $expirationDate);

                    $final_subscription[] = [
                        'id' => $subscription->id,
                        'name' => $product_name,
                        'expire_date' => $expirationDateFormatted,
                        'is_canceled' => $subscription->cancel_at_period_end
                    ];

                }
            }catch (\Exception $e){}
        }

        return view('update.your_subscriptions', compact('subscriptions', 'packages', 'final_subscription'));
    }

    public function cancel_subscriptions($subs_id)
    {
        $subs_id = Crypt::decrypt($subs_id);
        Stripe::setApiKey(env('STRIPE_SECRET'));

        $subscription = Subscription::update($subs_id, [
            'cancel_at_period_end' => true,
        ]);

        notify()->success('Subscription canceled successfully',"Success");
        return redirect()->back();

    }

    public function settings()
    {
        return view('settings');
    }

    public function rate()
    {
        $customer_info = Cookie::get('customer_info');
        $customer_info = json_decode($customer_info, true);
        $response = $this->apiService->callAPI('POST', 'api/rates/', [
            "id" => $customer_info['id'],
            "token" => Cookie::get('c_auth_token'),
            "action" =>  "rate_list",
            "object_where_params" => [
                "code" => "",
                "destination" => "",
                "gracetime" => "",
                "initial_increment" => "",
                "increment" => "",
                "country_code" => "",
            ],
            "device_id" => "",
            "start_limit" => "1",
            "end_limit" => "50",
        ]);
        if ($response['response_code'] == 400){
            return redirect()->route('logout');
        }

        $rates = $response['data'] ?? [];

        $response = $this->apiService->callAPI('POST', 'api/global_api/', [
            "action" => "get_country_list",
            "device_id" =>  "",
        ]);
        $country_list = [];
        if ($response['response_code'] == 200){
            $country_list = $response['data'] ?? [];
        }else{
            return redirect()->route('customer.login')->with('error', $response['error']);
        }

        return view('rate', compact('rates', 'country_list'));
    }

    public function check_rate(Request $request)
    {
	    $customer_info = Cookie::get('customer_info');
        $customer_info = json_decode($customer_info, true);
        $rates = [];
        $check = false;
        if ($request['code'] != '' ){
            $response = $this->apiService->callAPI('POST', 'api/rates/', [
                "id" => $customer_info['id'],
                "token" => Cookie::get('c_auth_token'),
                "action" =>  "rate_list",
                "object_where_params" => [
//                    "code" => $request['code'] != '' ? $request['code'] : '',
                    "destination" => "",
                    "gracetime" => "",
                    "initial_increment" => "",
                    "increment" => "",
                    "country_code" => $request['init_code'],
                ],
                "device_id" => "",
                "start_limit" => "1",
                "end_limit" => "10000000",
            ]);
//            dd([
//                "code" => $request['code'] != '' ? $request['code'] : '',
//                "destination" => "",
//                "gracetime" => "",
//                "initial_increment" => "",
//                "increment" => "",
//                "country_code" => 'usd',$request['init_code'],
//            ]);
//            echo json_encode($response['data']);exit;
            if ($response['response_code'] == 400){
                if ($response['error'] == 'Invalid key found.'){
                    return redirect()->route('logout');
                }
                notify()->error($response['error'],"Error");
                return redirect()->back();
            }
            $rates = $response['data'] ?? [];
            $rates = collect($rates)->where('pattern',$request['code'])->all();
            $check = true;
        }

        $init_code = $request['init_code'] != '' ? $request['init_code'] : 'US';
        $code = $request['code'] != '' ? $request['code'] : '1';

        return view('update.check_rate', compact('rates', 'init_code', 'check', 'code'));
    }

    public function apply_coupon(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'coupon_number' => 'required',
        ]);
        if ($validator->fails()){
            notify()->error('This coupon number filed is required.',"Error");
            return redirect()->route('dashboard');
        }
        $customer_info = Cookie::get('customer_info');
        $customer_info = json_decode($customer_info, true);

        $response = $this->apiService->callAPI('POST', 'api/user_refill_coupon/', [
            "id" => $customer_info['id'],
            "token" => Cookie::get('c_auth_token'),
            "action" =>  "refill_coupon_add",
            "coupon_number" => $request->coupon_number,
            "device_id" => "",
        ]);
        if ($response['response_code'] == 400){
            if ($response['error'] == 'Invalid key found.'){
                return redirect()->route('logout');
            }
            notify()->error($response['error'],"Error");
            return redirect()->back();
        }
        notify()->success($response['success'],"Success");
        return redirect()->back();
    }

    public function add_credit()
    {
        $customer_info = Cookie::get('customer_info');
        $customer_info = json_decode($customer_info, true);
        $response = CustomerLogic::get_balance($this->apiService, $customer_info['id']);
        if ($response['response_code'] != 200){
            if ($response['error'] == 'Account Not Found'){
                return redirect()->route('logout');
            }
            $balance = '';
            notify()->error($response['error'],"Error");

        }
        $balance = $response['data'];
        return view('update.add_credit', compact('balance'));
    }

    public function profile_update(Request $request)
    {

        $validator = Validator::make($request->all(), [
            'first_name' => 'required',
            'last_name' => 'required',
            'email' => 'required',
        ]);
        if ($validator->fails()){
            notify()->error('Filed must not be empty.',"Error");
            return redirect()->back();
        }
        $customer_info = Cookie::get('customer_info');
        $customer_info = json_decode($customer_info, true);

        $response = $this->apiService->callAPI('POST', 'api/user_profile/', [
            "id" => $customer_info['id'],
            "token" => Cookie::get('c_auth_token'),
            "action" =>  "update_profile",
            "company_name" => $request->company_name,
            "first_name" => $request->first_name,
            "last_name" => $request->last_name,
            "telephone_1" => $customer_info['telephone_1'],
            "email" => $request['email'],
            "timezone_id" => $customer_info['timezone_id'],
            "address_1" => $customer_info['address_1'],
            "address_2" => $customer_info['address_2'],
            "city" => $customer_info['city'],
            "province" => $customer_info['province'],
            "postal_code" => $customer_info['postal_code'],
            "tax_number" => "",
            "device_id" => "",
        ]);
        if ($response['response_code'] == 400){
            if ($response['error'] == 'Invalid key found.'){
                return redirect()->route('logout');
            }
            notify()->error($response['error'],"Error");
            return redirect()->back();
        }
        notify()->success($response['success'],"Success");
        return redirect()->back();
    }

    public function package_buy(Request $request)
    {
        if($request->has('q') && $request->input('q') != '' && !empty($request->input('q'))){
            $additionalData = json_decode(Crypt::decryptString($request->input('q')),true);
//            dump($request->all());
            $request->merge($additionalData);
//            unset($request->)
//            dd($request->all());
        }


        $postRequest    =   $request->all();
        if(!empty($postRequest['_token'])) unset($postRequest['_token']);
        $postRequest    =   json_encode($postRequest);
        $postRequest    = Crypt::encryptString($postRequest);
//        dd($postRequest);
        $credit_amount = null;
        if($request->has('type') && $request->input('type') == 'credit'){
            $product       = null;
            $credit_amount = $request->credit_amount;
        }
        else{
            $price = preg_replace('/[^0-9.]/', '',$request['price']);
            $spacePosition = strpos($request['price'], ' ');
            $price = substr($price, 0, $spacePosition);
            $response = $this->apiService->callAPI('GET', 'product-list.php', []);

            $product = collect($response)->where('name', $request->product_name)
                ->where('price', $price)
                ->where('free_minutes', $request->free_minutes)
                ->first();
        }

        $customer_info = Cookie::get('customer_info');
        $customer_info = json_decode($customer_info, true);

        $response = CustomerLogic::get_balance($this->apiService, $customer_info['id']);
        if ($response['response_code'] != 200){
            if ($response['error'] == 'Account Not Found'){
                return redirect()->route('logout');
            }
            $balance = '';
            notify()->error($response['error'],"Error");
        }
        $balance = $response['data'];


        $customer_info = Cookie::get('customer_info');
        $customer_info = json_decode($customer_info, true);
        $stripe_customer = StripeCustomer::where('customer_id', $customer_info['id'])->first();
        $paymentMethod = null;
        if ($stripe_customer) {
            Stripe::setApiKey(env('STRIPE_SECRET'));
            $customer = \Stripe\Customer::retrieve($stripe_customer->stripe_customer_id);

            $default_payment_method_id = $customer->invoice_settings->default_payment_method;
            if($default_payment_method_id){
                $paymentMethod = PaymentMethod::retrieve($default_payment_method_id);
            }
        }

        return view('package_buy', compact('product', 'balance','paymentMethod','credit_amount','postRequest'));
    }



    public function create_payment_intent(Request $request)
    {
//        dd($request->all());
        // Set Stripe API key
        Stripe::setApiKey(env('STRIPE_SECRET'));
        $customer_info      =   Cookie::get('customer_info');
        $customer_info      =   json_decode($customer_info, true);
        $stripe_customer    =   StripeCustomer::where('customer_id', $customer_info['id'])->first();

        // Retrieve the customer from Stripe
        try {
            $customer = \Stripe\Customer::retrieve($stripe_customer->stripe_customer_id);
        } catch (\Exception $e) {
            Log::info($e->getMessage(),$customer_info);
        }

        if($request->product_id){
            $amount_type =  'product';
            $response = $this->apiService->callAPI('GET', 'product-list.php', []);
            $product = collect($response)->where('id', $request->product_id)->first();
            $price  = $product['price'];
        }else{
            $amount_type =  'credit_amount';
            $price = $request->credit_amount;
        }
        // Create a PaymentIntent with Apple Pay as the payment method
        $paymentIntent = PaymentIntent::create([
            'amount' => round($price)*100, // Amount in cents
            'currency' => 'usd',
            'payment_method_types' => ['card'],
            'metadata' => [
                'amount_type' => $amount_type,
                'customer_id' => $customer_info['id'],
                'number' => $customer_info['number'],
                'type' => $customer_info['type'],
                'product_id' =>  @$product['id'] ?? '',
                'product_category' => @$product['product_category'] ?? '',
            ],
        ]);

        return response()->json([
            'name' => @$product ? @$product['name']:'Credit',
            'amount' => round($price)*100,
            'clientSecret' => $paymentIntent->client_secret,
        ]);
    }

    public function save_payment_method(Request $request)
    {
        Stripe::setApiKey(env('STRIPE_SECRET'));

        $customer_info = Cookie::get('customer_info');
        $customer_info = json_decode($customer_info, true);

        $response = $this->apiService->callAPI('POST', 'api/global_api/', [
            "action" => "get_country_list",
            "device_id" =>  "",
        ]);

        $unique_id = Str::random(6) . '-' . rand(1, 1000).'-'.uniqid();
        session()->put('unique_id', $unique_id);

        $country_list = $response['data'];
        $country = collect($country_list)->where('id', $customer_info['country_id'])->first()['iso3'];


        $country = $country == 'PAK' ?'US':$country;

        Stripe::setApiKey(env('STRIPE_SECRET'));
        header('Content-Type: application/json');

        $currency_code = 'USD';
        $currencies_not_supported_cents = ['BIF', 'CLP', 'DJF', 'GNF', 'JPY', 'KMF', 'KRW', 'MGA', 'PYG', 'RWF', 'UGX', 'VND', 'VUV', 'XAF', 'XOF', 'XPF'];

        $base_url = url('/');
        // Retrieve the Stripe token from the request
        $customer_info = Cookie::get('customer_info');
        $customer_info = json_decode($customer_info, true);
        // Create the payment method
        try {
            $countryCode = collect($country_list)->where('id', $customer_info['country_id'])->first()['iso'];
            $check_customer = StripeCustomer::where('customer_id', $customer_info['id'])->first();
            if (!empty($check_customer)){
                $customer = \Stripe\Customer::update($check_customer['stripe_customer_id'], [
                    'email' => $customer_info['email'],
                    'address' => [
                        'city' => $customer_info['city'],
                        'country' => $countryCode, // Pass the customer's country here
                        'line1' =>$customer_info['address_1'],
                        'line2' =>$customer_info['address_2'],
                        'postal_code' =>$customer_info['postal_code'],
                    ],
                ]);

            }
            else{
                $customer = \Stripe\Customer::create([
                    'email' => $customer_info['email'],
                    'address' => [
                        'city' => $customer_info['city'],
                        'country' => $countryCode, // Pass the customer's country here
                        'line1' =>$customer_info['address_1'],
                        'line2' =>$customer_info['address_2'],
                        'postal_code' =>$customer_info['postal_code'],
                    ],
                ]);

                $stripe_customer = new StripeCustomer();
                $stripe_customer->customer_id = $customer_info['id'];
                $stripe_customer->stripe_customer_id = $customer->id;
                $stripe_customer->save();
            }


            $customerId = $customer->id;
            $paymentMethodId = $request->input('payment_method_id');

            // Attach the payment method to the customer
            $paymentMethod = PaymentMethod::retrieve($paymentMethodId);
            $paymentMethod->attach(['customer' => $customerId]);

            // Update the customer to set this payment method as the default
            Customer::update($customerId, [
                'invoice_settings' => [
                    'default_payment_method' => $paymentMethodId,
                ],
            ]);
            return response()->json(['status' => 'success', 'message' => 'Payment done and Payment method added successfully']);
        } catch (\Exception $e) {
            return response()->json(['status' => 'error', 'message' => $e->getMessage()], 500);
        }

    }

    public function createProductAndPriceIfNeeded($product)
    {
        // Set Stripe API key
        Stripe::setApiKey(env('STRIPE_SECRET'));

        $productId = $product['id'];
        $priceAmount = round($product['price'])*100; // Amount in cents

        $productName = $product['name'];
        $priceAmount = round($product['price'])*100;
        $currency = 'usd';
        $intervalDays = $product['billing_days'];

        // Check if the product exists
        $product = null;
        $products = Product::all(['limit' => 100]); // Adjust the limit as necessary
        foreach ($products->data as $p) {
            if ($p->name === $productName) {
                $product = $p;
                break;
            }
        }

        if (!$product) {
            // If product does not exist, create it
            $product = Product::create([
                'name' => $productName, // Replace with actual product name
                'type' => 'service',
            ]);
        }

        // Check if the price exists for the product
        $prices = Price::all(['product' => $product->id]);
        $price = null;
        foreach ($prices->data as $p) {
            if ($p->unit_amount == $priceAmount && $p->currency == $currency && $p->recurring->interval == 'day' && $p->recurring->interval_count == $intervalDays) {
                $price = $p;
                break;
            }
        }

        if (!$price) {
            // Create a price if not found
            $price = Price::create([
                'product' => $product->id,
                'unit_amount' => $priceAmount,
                'currency' => $currency,
                'recurring' => ['interval' => 'day', 'interval_count' => $intervalDays],
            ]);
        }

        return $price->id;
    }

    public function package_buy_pay(Request $request)
    {
//        dd($request->all());
        if ($request->payment_method == ''){
            notify()->error('Please select a payment option', 'Error');
            return redirect()->route('dashboard');
        }
        $response = $this->apiService->callAPI('GET', 'product-list.php', []);

        $product = collect($response)->where('id', $request->product_id)
            ->first();

        $product_id = $request->product_id;
        if ($request->payment_method == 'account_balance' && $product['billing_type'] == 0){
            $customer_info = Cookie::get('customer_info');
            $customer_info = json_decode($customer_info, true);
            $response = $this->apiService->callAPI('POST', 'api/user_order/', [
                                                                                                    "id" => $customer_info['id'],
                                                                                                    "token" => Cookie::get('c_auth_token'),
                                                                                                    "action" =>  "order",
                                                                                                    "product_category" => $product['product_category'],
                                                                                                    "product_id" => $product['id'],
                                                                                                    "product_quantity" => 1,
                                                                                                    "device_id" => "",
                                                                                                ]);
            if ($response['response_code'] == 400){
                if ($response['error'] == 'Invalid key found.'){
                    return redirect()->route('logout');
                }
                notify()->error($response['error'],"Error");
                return redirect()->route('dashboard');
            }
            notify()->success($response['success'],"Success");
            return redirect()->route('dashboard');
        }
        else{
            if($request->has('payment_method_id') && !empty($request->input('payment_method_id')) ){
                $response = $this->apiService->callAPI('GET', 'product-list.php', []);

                $product = collect($response)->where('id', $request->product_id)->first();
                Stripe::setApiKey(env('STRIPE_SECRET'));
                $customer_info = Cookie::get('customer_info');
                $customer_info = json_decode($customer_info, true);
                $stripe_customer = StripeCustomer::where('customer_id', $customer_info['id'])->first();
                try {
// Retrieve the customer from Stripe
                    $customer = \Stripe\Customer::retrieve($stripe_customer->stripe_customer_id);

                    // Retrieve the default payment method of the customer
                    $defaultPaymentMethod = $customer->invoice_settings->default_payment_method;

                    if (!$defaultPaymentMethod) {
                        notify()->error('No default payment method found.',"Error");
                        return redirect(route('dashboard'));
                    }
                    if($product){
                        if($product['billing_type'] == 2){
                            $priceId = $this->createProductAndPriceIfNeeded($product);
                            $subscriptions = Subscription::all(['customer' => $customer->id, 'status' => 'active']);
//                        dd($subscriptions);
                            foreach ($subscriptions->data as $subscription) {
                                foreach ($subscription->items->data as $item) {
//                                dd(" {$item->plan->id} == $priceId ");
                                    if ($item->plan->id == $priceId) {
                                        notify()->success('You already have this active subscription.',"warning");
                                        return redirect(route('dashboard'));
                                    }
                                }
                            }
                            $subscription = Subscription::create([
                                'customer' => $customer->id,
                                'items' => [['price' => $priceId]],
                                'expand' => ['latest_invoice.payment_intent'],
                            ]);


//                            if ($paymentIntent->status == 'requires_action' || $paymentIntent->status == 'requires_payment_method') {
//                                return response()->json([
//                                    'status' => 'incomplete',
//                                    'error' => $paymentIntent->last_payment_error->message,
//                                ]);
//                            }
                            if($subscription->status == 'active'){
                                notify()->success('Subscription created successfully.',"success");
                            }else{
                                $paymentIntent = $subscription->latest_invoice->payment_intent;
                                notify()->error('Subscription Failed '.$paymentIntent->last_payment_error->message,"Error");
                            }
                            return redirect(route('dashboard'));

                        }
                        else{
                            PaymentIntent::create([
                                'amount' => round($product['price'])*100, // Amount in cents
                                'currency' => 'usd',
                                'customer' => $customer->id,
                                'payment_method' => $defaultPaymentMethod,
                                'off_session' => true,
                                'confirm' => true,
                                'metadata' => [
                                    'amount_type' => 'product',
                                    'customer_id' => $customer_info['id'],
                                    'number' => $customer_info['number'],
                                    'type' => $customer_info['type'],
                                    'product_id' =>  @$product['id'] ?? '',
                                    'product_category' => @$product['product_category'] ?? '',
                                ],
                            ]);
                        }
                    }
                    else{
                        if($request->credit_amount < 1){
                            notify()->success('invalid Amount.',"warning");
                            return redirect(route('dashboard'));
                        }
                        PaymentIntent::create([
                            'amount' => round($request->credit_amount)*100, // Amount in cents
                            'currency' => 'usd',
                            'customer' => $customer->id,
                            'payment_method' => $defaultPaymentMethod,
                            'off_session' => true,
                            'confirm' => true,
                            'metadata' => [
                                'amount_type' => 'credit_amount',
                                'customer_id' => $customer_info['id'],
                                'number' => $customer_info['number'],
                                'type' => $customer_info['type'],
                            ],
                        ]);
                        // Log::info("BEFORE ".$request->credit_amount.json_encode($customer_info));
                        // CustomerLogic::add_balance($this->apiService, $customer_info, $request->credit_amount);
                        // Log::info("AFTER ".$request->credit_amount.json_encode($customer_info));
                        notify()->success('Credit Balance Payment successful.',"success");
                        return redirect(route('dashboard'));
                    }
                    notify()->success('Payment successful.',"success");
                    return redirect(route('dashboard'));
                } catch (ApiErrorException $e) {
                    // Handle Stripe API errors
                    notify()->error($e->getMessage(),"Error");
                    return redirect(route('dashboard'));

                } catch (\Exception $e) {
                    // Handle other exceptions
                    notify()->error($e->getMessage(),"Error");
                    return redirect(route('dashboard'));
                }

            }else{
                return redirect()->route('stripe_payment', Crypt::encrypt($product_id));
            }
        }

    }


    public function add_payment_setting(Request $request)
    {

        $dq = null;
        if($request->has('q') && $request->input('q') != '' && !empty($request->input('q'))){
            $q = $request->input('q');
            $dq = Crypt::decryptString($request->input('q'));
        }
        return view('update.add_payment_setting',compact('dq'));
    }

    public function save_payment_setting(Request $request)
    {
        $q = null;
        if($request->has('q') && $request->input('q') != '' && !empty($request->input('q'))){
            $q = $request->input('q');
        }
        Stripe::setApiKey(env('STRIPE_SECRET'));

        $customer_info = Cookie::get('customer_info');
        $customer_info = json_decode($customer_info, true);

        $response = $this->apiService->callAPI('POST', 'api/global_api/', [
            "action" => "get_country_list",
            "device_id" =>  "",
        ]);

        $unique_id = Str::random(6) . '-' . rand(1, 1000).'-'.uniqid();
        session()->put('unique_id', $unique_id);

        $country_list = $response['data'];

        $country = collect($country_list)->where('id', $customer_info['country_id'])->first()['iso3'];

        $country = $country == 'PAK' ?'US':$country;

        Stripe::setApiKey(env('STRIPE_SECRET'));
        header('Content-Type: application/json');

        $currency_code = 'USD';
        $currencies_not_supported_cents = ['BIF', 'CLP', 'DJF', 'GNF', 'JPY', 'KMF', 'KRW', 'MGA', 'PYG', 'RWF', 'UGX', 'VND', 'VUV', 'XAF', 'XOF', 'XPF'];

        $base_url = url('/');
        // Retrieve the Stripe token from the request
        $customer_info = Cookie::get('customer_info');
        $customer_info = json_decode($customer_info, true);
        // Create the payment method
        try {
            $countryCode = collect($country_list)->where('id', $customer_info['country_id'])->first()['iso'];
            $check_customer = StripeCustomer::where('customer_id', $customer_info['id'])->first();
            if (!empty($check_customer)){
                $customer = \Stripe\Customer::update($check_customer['stripe_customer_id'], [
                    'email' => $customer_info['email'],
                    'address' => [
                        'city' => $customer_info['city'],
                        'country' => $countryCode, // Pass the customer's country here
                        'line1' =>$customer_info['address_1'],
                        'line2' =>$customer_info['address_2'],
                        'postal_code' =>$customer_info['postal_code'],
                    ],
                ]);

            }
            else{
                $customer = \Stripe\Customer::create([
                    'email' => $customer_info['email'],
                    'address' => [
                        'city' => $customer_info['city'],
                        'country' => $countryCode, // Pass the customer's country here
                        'line1' =>$customer_info['address_1'],
                        'line2' =>$customer_info['address_2'],
                        'postal_code' =>$customer_info['postal_code'],
                    ],
                ]);

                $stripe_customer = new StripeCustomer();
                $stripe_customer->customer_id = $customer_info['id'];
                $stripe_customer->stripe_customer_id = $customer->id;
                $stripe_customer->save();
            }

            $paymentMethod = PaymentMethod::create([
                'type' => 'card',
                'card' => ['token' => $request->input('stripeToken')],
                'billing_details' => [
                    'email' =>  $customer_info['email'],
                    'address' => [
                        'city' => $customer_info['city'],
                        'country' => $countryCode, // Pass the customer's country here
                        'line1' =>$customer_info['address_1'],
                        'line2' =>$customer_info['address_2'],
                        'postal_code' =>$customer_info['postal_code'],
                    ],
                ],
            ]);
            $paymentMethod->attach(['customer' => $customer->id]);
            if($request->preferredMethod && $request->preferredMethod == 1){
                \Stripe\Customer::update($customer->id, [
                    'invoice_settings' => [
                        'default_payment_method' => $paymentMethod->id
                    ],
                ]);
            }

            if($request->has('makePayment') && $request->input('makePayment') == 1){

                $response = $this->apiService->callAPI('GET', 'product-list.php', []);

                $product = collect($response)->where('id', $request->product_id)->first();
                Stripe::setApiKey(env('STRIPE_SECRET'));
                $customer_info = Cookie::get('customer_info');
                $customer_info = json_decode($customer_info, true);
                $stripe_customer = StripeCustomer::where('customer_id', $customer_info['id'])->first();

                // Retrieve the customer from Stripe
                $customer = \Stripe\Customer::retrieve($stripe_customer->stripe_customer_id);

                // Retrieve the default payment method of the customer
                $defaultPaymentMethod = $customer->invoice_settings->default_payment_method;

                if (!$defaultPaymentMethod) {
                    $defaultPaymentMethod = $paymentMethod->id;
//                    return response()->json(['status' => 'error', 'message' => 'No default payment method found.']);
//                    notify()->success('No default payment method found.',"Error");
//                    return redirect()->back();
                }
                if($product){
                    if($product['billing_type'] == 2){
                        $priceId = $this->createProductAndPriceIfNeeded($product);
                        $subscriptions = Subscription::all(['customer' => $customer->id, 'status' => 'active']);
//                        dd($subscriptions);
                        foreach ($subscriptions->data as $subscription) {
                            foreach ($subscription->items->data as $item) {
//                                dd(" {$item->plan->id} == $priceId ");
                                if ($item->plan->id == $priceId) {
                                    return response()->json(['status' => 'error', 'message' => 'Customer already has this subscription.']);
                                    notify()->success('Customer already has this subscription.',"warning");
                                    return redirect(route('dashboard'));
                                }
                            }
                        }
                        Subscription::create([
                            'customer' => $customer->id,
                            'items' => [['price' => $priceId]],
                            'expand' => ['latest_invoice.payment_intent'],
                        ]);

                    }
                    else{
                        PaymentIntent::create([
                            'amount' => round($product['price'])*100, // Amount in cents
                            'currency' => 'usd',
                            'customer' => $customer->id,
                            'payment_method' => $defaultPaymentMethod,
                            'off_session' => true,
                            'confirm' => true,
                            'metadata' => [
                                'amount_type' => 'product',
                                'customer_id' => $customer_info['id'],
                                'number' => $customer_info['number'],
                                'type' => $customer_info['type'],
                                'product_id' =>  @$product['id'] ?? '',
                                'product_category' => @$product['product_category'] ?? '',
                            ],
                        ]);
                    }
                }
                else{
                    if($request->credit_amount < 1){
                        return response()->json(['status' => 'error', 'message' => 'invalid Amount.']);
                        notify()->success('invalid Amount.',"warning");
                        return redirect()->back();
                    }
                    PaymentIntent::create([
                        'amount' => round($request->credit_amount)*100, // Amount in cents
                        'currency' => 'usd',
                        'customer' => $customer->id,
                        'payment_method' => $defaultPaymentMethod,
                        'off_session' => true,
                        'confirm' => true,
                        'metadata' => [
                            'amount_type' => 'credit_amount',
                            'customer_id' => $customer_info['id'],
                            'number' => $customer_info['number'],
                            'type' => $customer_info['type'],
                        ],
                    ]);
                    // Log::info("BEFORE ".$request->credit_amount.json_encode($customer_info));
                    // CustomerLogic::add_balance($this->apiService, $customer_info, $request->credit_amount);
                    // Log::info("BEFORE ".$request->credit_amount.json_encode($customer_info));
                    return response()->json(['status' => 'success', 'message' => 'Payment successful.']);
                    return redirect(route('dashboard'));
                }
            }
            return response()->json(['status' => 'success', 'message' => 'Payment successful.']);
        } catch (\Exception $e) {
            return response()->json(['status' => 'error', 'message' => $e->getMessage()], 500);
        }

    }

    public function payment_setting(Request $request)
    {
        $q = null;
        if($request->has('q') && $request->input('q') != '' && !empty($request->input('q'))){
            $q = $request->input('q');
        }
        $update = false;
        $customer_info = Cookie::get('customer_info');
        $customer_info = json_decode($customer_info, true);
        $stripe_customer = StripeCustomer::where('customer_id', $customer_info['id'])->first();

        if ($stripe_customer) {
            Stripe::setApiKey(env('STRIPE_SECRET'));
            $paymentMethods = \Stripe\PaymentMethod::all([
                'customer' => $stripe_customer->stripe_customer_id,
                'type' => 'card', // Filter by card payment methods
            ]);

            $customer = \Stripe\Customer::retrieve($stripe_customer->stripe_customer_id);
            $default_payment_method_id = $customer->invoice_settings->default_payment_method;
            $payment_method_id = null;
            if($request->has('id') && collect($paymentMethods)->where('id',$request->input('id'))->first()){
                $payment_method_id = $request->input('id');
                $paymentMethods    = collect($paymentMethods)->where('id',$request->input('id'))->all();
                $update = true;
            }

        }else{
            $paymentMethods = [];
            $default_payment_method_id = null;
        }

        return view('update.payment_setting', compact('paymentMethods', 'default_payment_method_id', 'stripe_customer','update','payment_method_id','q'));
    }



    public function set_default(Request $request,$pay_id, $cus_id)
    {
        Stripe::setApiKey(env('STRIPE_SECRET'));
        $customer_id = $cus_id;

        // Replace 'payment_method_id' with the ID of the payment method you want to set as default
        $payment_method_id = $pay_id;

        try {
            // Update the customer to set the default payment method
            $customer = \Stripe\Customer::update($customer_id, [
                'invoice_settings' => [
                    'default_payment_method' => $payment_method_id,
                ],
            ]);

            notify()->success('Preferred Payment method set successfully ',"Success");
            if($request->has('q') && $request->input('q') != '' && !empty($request->input('q'))){
                return redirect(route('package_buy_get')."?q=".$request->input('q'));
            }
            return redirect(route('payment_setting'));

        }catch (\Exception $e){
            notify()->error($e->getMessage(),"Error");
            return redirect()->back();
        }

    }
    public function drop_payment_setting(Request $request,$payment_method_id)
    {
        $q = null;
        if($request->has('q') && $request->input('q') != '' && !empty($request->input('q'))){
            $q = $request->input('q');
        }
        $customer_info = Cookie::get('customer_info');
        $customer_info = json_decode($customer_info, true);
        $stripe_customer = StripeCustomer::where('customer_id', $customer_info['id'])->first();
        Stripe::setApiKey(env('STRIPE_SECRET'));

        $customer = \Stripe\Customer::retrieve($stripe_customer->stripe_customer_id);
        if(!$customer){
            notify()->error('Invalid request',"Error");
            if($request->has('q') && $request->input('q') != '' && !empty($request->input('q'))){
                return redirect(route('package_buy_get')."?q=".$request->input('q'));
            }
            return redirect()->back();
        }
        if($customer->invoice_settings->default_payment_method == $payment_method_id){
            notify()->error('Preferred Payment Method cannot be deleted.',"Error");
            if($request->has('q') && $request->input('q') != '' && !empty($request->input('q'))){
                return redirect(route('package_buy_get')."?q=".$request->input('q'));
            }
            return redirect()->back();
        }

        try {
            $paymentMethod = PaymentMethod::retrieve($payment_method_id);
            $paymentMethod->detach();
            notify()->success('Payment method deleted  successfully ',"Success");
            if($request->has('q') && $request->input('q') != '' && !empty($request->input('q'))){
                return redirect(route('package_buy_get')."?q=".$request->input('q'));
            }
            return redirect(route('payment_setting'));

        }catch (\Exception $e){
            notify()->error($e->getMessage(),"Error");
            if($request->has('q') && $request->input('q') != '' && !empty($request->input('q'))){
                return redirect(route('package_buy_get')."?q=".$request->input('q'));
            }
            return redirect()->back();
        }

    }

    public function update_apps()
    {
//        $customer_info = Cookie::get('customer_info');
//        $customer_info = json_decode($customer_info, true);
//        dd($customer_info['dir_params']['vm-password']);
        return view('update_apps');
    }

    public function handleProvisioning(Request $request)
    {
        $data = $request->all();
        $cloudUsername = isset($data['cloud_username']) ? trim(strtolower($data['cloud_username'])) : '';
        $cloudPassword = isset($data['cloud_password']) ? $data['cloud_password'] : '';
        $cloudId = isset($data['cloud_id']) ? $data['cloud_id'] : '';
        /*if ($cloudId != env('CLOUD_ID')){
            return abort(401);
        }*/

        $body = [
            "username" => $cloudUsername,
            "password" => $cloudPassword,
            "device_id" =>  "645C18B55-26C-4504-AADF-034BDFE1AFEA1",
            "callkit_token" =>  "BE6CAF775FFC2C1AAD28D9992E467156F044D68D21C59E4973C3A692DACAB03C",
            "apns_token" =>  "63c7620a2c5ce0a1717850ecb559fb994c57bc6180f49c2e815efab09421f924",
            "mobile_type"=> "android"
        ];

        $response = $this->apiService->callAPI('POST', 'api/login/', $body);

        if ($response['status'] == true){
            $user = $response['data'];
            $sip_device_res = $this->apiService->callAPI('POST', 'api/user_sip_device/', [
                "id" => $user['id'],
                "token" => $user['token'],
                "action" =>  "user_sipdevices_list",
                "start_limit" => "1",
                "end_limit" => "2",
                "device_id" => "",
                "object_where_params" => [
                    "username" => ""
                ]
            ]);
            if ($sip_device_res['response_code'] == 400){
                if ($response['error'] == 'Invalid key found.'){
                    return redirect()->route('logout');
                }
                return abort(401);
            }
            if (count($sip_device_res['data']) > 0){
                $sip_device = $sip_device_res['data'][0];

                    $xmlContent = '
                        <account>
                            <title>'.$cloudUsername.'</title>
                            <cloud_username>'.$cloudUsername.'</cloud_username>
                            <cloud_password>'.$cloudPassword.'</cloud_password>
                            <username>'.$sip_device['username'].'</username>
                            <password>'.$sip_device['password'].'</password>
                            <host>'.env('SIP_DOMAIN').'</host>
                            <extProvInterval>3600</extProvInterval>
                            <genericBalanceCheckUrl>'.url('check-balance').'?username=%account[username]%</genericBalanceCheckUrl>
                            <extProvUrl>'.url('provisioning').'?cloud_username=%account[cloud_username]%&amp;cloud_password=%account[cloud_password]%&amp;cloud_id='.$cloudId.'</extProvUrl>
                        </account>';

                // Return XML response
                return Response::make($xmlContent)->header('Content-Type', 'text/xml');
            }else{
                abort(404);
            }
        }else{
            abort(403);
        }
    }

    public function check_balance(Request $request)
    {
        $username = $request['username'];
        $response = CustomerLogic::get_customer_list($this->apiService, $username);
        if ($response['response_code'] != 200){
            if ($response['error'] == 'Account Not Found'){
                abort(401);
            }
            abort(404);
        }

        if (!isset($response['data'][0]['accountid'])){
            abort(404);
        }
        $customer_id = $response['data'][0]['accountid'];
        $balance_res = CustomerLogic::get_balance($this->apiService, $customer_id);
        if ($balance_res['response_code'] != 200){
            if ($balance_res['error'] == 'Account Not Found'){
                return abort(401);
            }
            return abort(404);
        }

        $balance_data = $balance_res['data'];
        $parts = explode(' ', $balance_data);

        $balance = $parts[0]; // Balance
        $currency = $parts[1]; // Currency
        $response = '<response>
                        <balanceString>'.$currency.' '.$balance.'</balanceString>
                        <balance>'.$balance.'</balance>
                        <currency>'.$currency.'</currency>
                    </response>';
        return Response::make($response)->header('Content-Type', 'text/xml');

    }


    public function privacy_policy()
    {
        return view('update.privacy_policy');
    }

    public function copyright()
    {
        return view('update.copyright');
    }

    public function support(Request $request)
    {
        if ($request->isMethod('post')){
            $data = [
                'name' => $request->name,
                'email' => $request->email,
                'phone' => $request->phone,
                'message' => $request->message,
	    ];

	    $email = 'info@avionuas.com';

            Mail::send('emails.contact', ['data' => $data], function ($messege) use ($email){
                $messege->to($email)->subject('Support Mail From Alotel');
            });



            notify()->success('Mail send successfully',"Success");
            return redirect()->back();
        }
        return view('support');
    }

    public function stripe(Request $request)
    {
        if ($request->isMethod('post')){
            dd($request->all());
        }else{
            // Set your Stripe API key
            Stripe::setApiKey('sk_test_51OvmI12Nt0ZHEStjvnDLBqZ7e0llkzr4mSgwlM5vDYnd7fwC3a26V3gznYU1vhFDszBKDksVKM9xX0z6c5UZLFXQ00RE49e64L');

            $customerId = 'customer_id_here'; // Replace 'customer_id_here' with the actual customer ID
            $customer = \Stripe\Customer::retrieve('cus_PxdS6oydGnPkp1');

            //return$payment_method = $customer->sources->retrieve('pi_3P7iBY2Nt0ZHEStj0c2D9B8V');
            $charges = Charge::all(['customer' => $customer['id']]);

            return $charges;
            view('stripe.payment');
        }
    }

     public function help()
    {
        return view('update.help');
    }




    public function notifications(){
        $response = $this->apiService->callAPI('GET', 'customers.php', []);
        dd($response);
        return view('update.notifications');
    }

}
