<?php

namespace App\Http\Controllers;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Str;

use \stdClass;
use Carbon\Carbon;
use DateTime;

use App\Helpers\Helper;
use App\Models\BodyType;
use App\Models\Ethnicity;
use App\Models\PrivateGalleryAccess;
use App\Models\User;
use App\Models\UserSeeking;
use App\Models\Relationship;
use App\Models\UserMeetup;
use App\Models\UserReport;
use App\Models\GroupMember;
use App\Models\Tap;
use App\Models\Group;
use App\Models\ChatList;
use App\Models\ChatMessage;
use App\Models\Notification;
use App\Models\SavedPhrase;


class InboxController extends Controller
{
    
    public function get_all_taps(Request $request)
    {
        if(isset($request->userId) ){
                $userTaps = Tap::where('tapping_to_user_id', $request->userId)->get();
                    if(sizeof($userTaps) > 0){
                        foreach($userTaps as $k){
                            

                            $tapByUser = User::where('users_id', $k->tapping_by_user_id)->first();

                            //for online status
                            if($tapByUser->online == 'Yes')
                                $k->is_online = true;
                            else
                                $k->is_online = false;

                            $k->user_name = $tapByUser->user_name;

                            if($tapByUser->profile_picture != null && $tapByUser->profile_picture != "")
                                $k->profile_picture = config('base_urls.profile_images_base_url').$tapByUser->profile_picture;
                            else
                                $k->profile_picture = ""; 

                            $k->formatted_time =  Carbon::parse($k->created_at)->format('g:i a');

                        }                          
                        $response = Helper::success(200, 'success', $userTaps);

                    }else{
                        $response = Helper::failed(404, 'error', 'No Taps found');
                    }
        }else{
            $response = Helper::failed(404, 'error', 'All fields are needed');
        }

        return Helper::custom_response($response);
    }

    public function destroy(Request $request)
    {
        
        if (isset($request->tap_id))
        {
                $id=$request->tap_id;
                $Tap = Tap::find($id);
            if($Tap)
            {
                $Tap->delete();
                return response()->json(['message'=>'Tap Deleted Successfully'],200);
            }
            else
            {
                return response()->json(['message'=>'Tap Not Found'],404);
            }
        }
        else
        {
            return response()->json(['message'=>'Tap ID Not Found'],404);

        }
        
    }

    public function Delete(Request $request)
    {
        
        if (isset($request->chat_message_id))
        {
                $id=$request->chat_message_id;
                $ChatMessage = ChatMessage::find($id);
            if($Tap)
            {
                $Tap->delete();
                return response()->json(['message'=>'Massage Deleted Successfully'],200);
            }
            else
            {
                return response()->json(['message'=>'Massage Not Found'],404);
            }
        }
        else
        {
            return response()->json(['message'=>'Massage ID Not Found'],404);

        }
        
    }

    public function get_inbox_list(Request $request)
    {
        if(isset($request->userId) ){
            $user_id = $request->userId;
            $mainDataset = array(); 

            //Chat list
            $chatList = ChatList::where('receiver_id',$user_id)->orderBy('chat_list_id','DESC')->get();

            if(sizeof($chatList) > 0){
                foreach($chatList as $k){
                    $senderDetails = User::where('users_id',$k->sender_id)->first();
                    $badgetCount = ChatMessage::where([ ['sender_id', $k->sender_id], ['receiver_id', $request->userId], ['status', 'Unread'] ])->count();
                    $recentMessageDetails = ChatMessage::where([ ['sender_id', $k->sender_id], ['receiver_id', $request->userId] ])->orderBy('created_at', 'DESC')->first();

                    if($recentMessageDetails)
                       $recentMessage = json_decode($recentMessageDetails->message);
                    else
                       $recentMessage = "";


                    $customObj = new stdClass();
                    $customObj->item_type = 'ChatItem';
                    $customObj->notification_id = $k->notification_id;
                    $customObj->sender_id = $k->sender_id;
                    $customObj->receiver_id = $k->receiver_id;
                    $customObj->sender_name = $senderDetails->user_name;
                    if($senderDetails->profile_picture != null && $senderDetails->profile_picture != "")
                        $customObj->profile_picture = config('base_urls.profile_images_base_url').$senderDetails->profile_picture;
                    else
                        $customObj->profile_picture = "";

                    $customObj->badge_count = $badgetCount;
                    $customObj->notification_type = null;
                    $customObj->group_id = null;
                    $customObj->message = $recentMessage;
                    $customObj->created_at = $k->created_at;
                    $customObj->formatted_time = Carbon::parse($k->created_at)->format('h:i a');
                    $customObj->status = $k->status;

                    //for online status
                    if($senderDetails->online == 'Yes')
                        $customObj->is_online = true;
                    else
                        $customObj->is_online = false;

                    array_push($mainDataset, $customObj);
                }
            }

            //Notification list
            $notificationList = Notification::where('receiver_users_id',$user_id)->orderBy('notification_id','DESC')->get();

            if(sizeof($notificationList) > 0){
                foreach($notificationList as $k){
                    $senderDetails = User::where('users_id',$k->sender_users_id)->first();

                    $customObj = new stdClass();
                    $customObj->item_type = 'NotificationItem';
                    $customObj->notification_id = $k->notification_id;
                    $customObj->sender_id = $k->sender_users_id;
                    $customObj->receiver_id = $k->receiver_users_id;
                    
                    $customObj->notification_type = $k->notification_type;
                    $customObj->group_id = $k->group_id; 
                    $customObj->message = $k->message;
                    $customObj->created_at = $k->datetime;
                    $customObj->formatted_time = Carbon::parse($k->datetime)->format('h:i a');
                    $customObj->status = $k->status;

                      //for online status
                      if($senderDetails->online == 'Yes')
                            $customObj->is_online = true;
                        else
                            $customObj->is_online = false;

                    array_push($mainDataset, $customObj);
                }

                foreach($mainDataset as $k){
                   if($k->group_id != null && $k->item_type == 'NotificationItem'){
                        $groupDetails = Group::with('category')->where('group_id', $k->group_id)->first();
                              $k->sender_name = $groupDetails->category->category;

                                if($groupDetails->cover_photo != null && $groupDetails->cover_photo != "")
                                    $k->profile_picture = config('base_urls.group_cover_photo_base_url').$groupDetails->cover_photo;
                                else
                                    $k->profile_picture = "";

                            if($k->notification_type == 'GroupCanceled'){
                                $badgeCount = Notification::where([ ['notification_type', 'GroupCanceled'], ['receiver_users_id', $request->userId], ['status', 'Unread'] ])->count();
                                 $k->badge_count = $badgeCount;
                              }
                           elseif($k->notification_type == 'GroupJoinRequest' && $k->status != 'Responded'){
                                   $badgeCount = Notification::where([ ['notification_type', 'GroupJoinRequest'], ['receiver_users_id', $request->userId], ['status', 'Unread'] ])->count();
                                    $k->badge_count = $badgeCount;
                               }
                           elseif($k->notification_type == 'GroupJoinAccept' && $k->status != 'Responded'){
                               $badgeCount = Notification::where([ ['notification_type', 'GroupJoinAccept'], ['receiver_users_id', $request->userId], ['status', 'Unread'] ])->count();
                                $k->badge_count = $badgeCount;
                             }
                             elseif($k->notification_type == 'GroupInviteRequest' && $k->status != 'Responded'){
                                $badgeCount = Notification::where([ ['notification_type', 'GroupInviteRequest'], ['receiver_users_id', $request->userId], ['status', 'Unread'] ])->count();
                                 $k->badge_count = $badgeCount;
                              }
                              elseif($k->notification_type == 'GroupInviteAccept' && $k->status != 'Responded'){
                                $badgeCount = Notification::where([ ['notification_type', 'GroupInviteAccept'], ['receiver_users_id', $request->userId], ['status', 'Unread'] ])->count();
                                 $k->badge_count = $badgeCount;
                              }
                   }

           

                }
            }

            $mainDataset = array_values(collect($mainDataset)->sortByDesc('created_at')->toArray());

            if(sizeof($mainDataset) > 0)
                     $response = Helper::success(200, 'success', $mainDataset);
              else
                    $response = Helper::failed(404, 'error', 'No notifications found');

        }else
            $response = Helper::failed(404, 'error', 'All fields are needed');
        

        return Helper::custom_response($response);
    }


    
  /*** CHAT ***/
  public function chat(Request $req){
    $response = array();
    $data = json_decode(file_get_contents("php://input"));

    if (isset($req->requestType)) {
        $request_type = $req->requestType;
                switch ($request_type) {

                        case "startChat":
                            if(isset($data->userId) && isset($data->otherUserId)){
                                    $check_request = ChatList::where([ ['sender_id', $req->userId], ['receiver_id', $req->otherUserId]])
                                                                ->orWhere([ ['sender_id', $req->otherUserId], ['receiver_id', $req->userId]])->count();
                                     if($check_request > 0)
                                             $response = Helper::failed(404, 'error', 'Chat already started');
                                          else{
                                                $data_save = array(
                                                    'sender_id'=> $req->userId,
                                                    'receiver_id'=> $req->otherUserId,
                                                    'date_request'=> date('Y-m-d'),
                                                    'created_at' => Carbon::now(),
                                                    'status'=> 'Active'
                                                );
                                                $requestSend = ChatList::insert($data_save);

                                                if($requestSend)
                                                    $response = Helper::success(200, 'success', 'Chat Started');
                                                 else 
                                                    $response = Helper::failed(404, 'error', 'Error in starting chat');
                                            }
                                        } else
                                            $response = Helper::failed(404, 'error', 'All fields are needed');
                                    break;   


                                          case "sendMessage":
                                            
                                                    $message_details = array(
                                                        'sender_id'=> $req->userId,
                                                        'receiver_id'=> $req->otherUserId,
                                                        'message'=>  json_encode($req->content) ,
                                                        'message_type'=> $req->messageType,
                                                        'send_date'=> date('Y-m-d'),
                                                        'send_time'=> date('H:i:s'),
                                                        'created_at'=> date('Y-m-d H:i:s'),
                                                        'status'=> 'Unread'
                                                    );

                                                            $insertedId = ChatMessage::insertGetId($message_details);
                                                            if($insertedId){
                                                                    $messageDetails = ChatMessage::where('chat_message_id', $insertedId)->first();
                                                                      $messageDetails->message = json_decode($messageDetails->message);

                                                                    if($messageDetails->message_type == 'attachment')
                                                                        $messageDetails->message = config('base_urls.chat_attachments_base_url').$messageDetails->message;
                                                                         
                                                                    $response = Helper::success(200, 'success', $messageDetails);

                                                            
                                                    } else
                                                        $response = Helper::failed(500, 'error', 'Something wrong in insert query');

                                            break;

                                       
                                            case "getMessages":
                                                if(isset($req->userId) && isset($data->otherUserId)){
                                                            $chat_array =array();
                                                            $day_array =array();
                                                          
                                                            $result = ChatMessage::where([
                                                                                            ['sender_id',$req->otherUserId],    
                                                                                            ['receiver_id', $req->userId]
                                                                                          ])->update(array('status' => 'Read'));  

                                                                           $all_chat = ChatMessage::where([
                                                                                                        ['sender_id',$req->userId],
                                                                                                        ['receiver_id',$req->otherUserId],
                                                                                                    ])->orWhere([
                                                                                                        ['sender_id',$req->otherUserId],
                                                                                                        ['receiver_id',$req->userId],
                                                                                                    ])->orderBy('chat_message_id','ASC')->get();
                                                        if(sizeof($all_chat) > 0){
                                                            foreach($all_chat as $key => $chat){

                                                                $chat->message = json_decode($chat->message);

                                                                $day = Helper::get_day_difference($chat->send_date);

                                                                if (in_array($day, $day_array, TRUE))
                                                                    $get_data['date']= '';
                                                                else {
                                                                    array_push($day_array, $day);
                                                                    $get_data['date']= $day;
                                                                } 

                                                                if($data->userId == $chat->receiver_id){
                                                                    $get_data['userId'] = $data->otherUserId;
                                                                    $get_data['time'] =  date('h:i A',strtotime($chat->send_time));
                                                                    $get_data['msgType'] = $chat->message_type;

                                                                    if($chat->message_type=='attachment'){
                                                                            $attachment = config('base_urls.chat_attachments_base_url') . $chat->message;
                                                                            $get_data['message'] = $attachment;
                                                                            }
                                                                            else{
                                                                                $get_data['message'] = $chat->message;
                                                                            }
                                                                        array_push($chat_array, $get_data);
                                                            }
                                                            else if($data->userId == $chat->sender_id ){
                                                                    $get_data['userId']= $data->userId;
                                                                    $get_data['time'] =   date('h:i A',strtotime($chat->send_time));
                                                                    $get_data['msgType'] = $chat->message_type;

                                                                    if($chat->message_type =='attachment'){
                                                                            $attachment = config('base_urls.chat_attachments_base_url') . $chat->message;
                                                                            $get_data['message'] = $attachment;
                                                                            }
                                                                            else{
                                                                                $get_data['message'] = $chat->message;
                                                                            }
                                                                        array_push($chat_array, $get_data);
                                                                }

                                                                    if(!empty($chat_array)){
                                                                                     $result = ChatMessage::where([
                                                                                                                    ['sender_id',$req->otherUserId],
                                                                                                                    ['receiver_id',$req->userId]
                                                                                                                ])->update(array('status'=>'Read'));
                                                                    }
                                                            }

                                                            foreach($chat_array as $key => $k){
                                                               
                                                                    $senderDetails = User::where('users_id',$k['userId'])->where('status','Active')->first();
                                                                    $chat_array[$key]['name']     =  $senderDetails->user_name;
    
                                                                    if($senderDetails->profile_picture != '' or $senderDetails->profile_picture != null)
                                                                        $chat_array[$key]['profile_pic']     = config('base_urls.profile_images_base_url') . $senderDetails->profile_picture;
                                                                     else
                                                                        $chat_array[$key]['profile_pic']     = '';

                                                            }
                                                            $response['chat_messages'] = $chat_array;

                                                            if($chat_array)
                                                                $response = Helper::success(200, 'success', $chat_array);
                                                            else
                                                                $response = Helper::success(500, 'error', 'Error in chat array');
                                                            
                                                        } else
                                                                 $response = Helper::success(404, 'error', 'No chat found');
                                                        
                                                } else
                                                        $response = Helper::success(404, 'error', 'All fields are needed');
                                            break;


                        case "updateMessages":
                            if(isset($req->userId) && isset($data->otherUserId)){
                                $user_id = $req->userId;
                                $other_user_id 	= $data->otherUserId;

                                $chat_array =array();
                                     $all_chat = ChatMessage::where([
                                                                            ['sender_id', $other_user_id],
                                                                            ['receiver_id',$user_id],
                                                                            ['status','Unread']
                                                                                ])->orderBy('chat_message_id', 'ASC')->get();
                                    if(sizeof($all_chat) > 0){
                                            foreach($all_chat as $chat){
                                                $chat->message = json_decode($chat->message);

                                                $get_data['userId']= $other_user_id;
                                                $get_data['time'] =  date('h:i A',strtotime($chat->send_date));
                                                $get_data['msgType'] = $chat->message_type;

                                                if($chat->message_type =='attachment'){
                                                    $image = config('base_urls.chat_attachments_base_url') . $chat->message;
                                                    $get_data['message'] = $image;
                                                }
                                                else
                                                    $get_data['message'] = $chat->message;
                                                    

                                                    array_push($chat_array, $get_data);
                                            } 
                                            if(!empty($chat_array)){
                                                       $result = ChatMessage::where([
                                                                                            ['sender_id',$other_user_id],
                                                                                            ['receiver_id',$user_id]
                                                                                        ])->update(array('status'=>'Read'));
                                            }
                                                   
                                            $chat_length 	 = ChatMessage::where([
                                                                                                ['sender_id', $user_id],
                                                                                                ['receiver_id',$other_user_id]
                                                                                                ])->orWhere([
                                                                                                    ['sender_id', $other_user_id],
                                                                                                    ['receiver_id',$user_id]
                                                                                                ])->orderBy('chat_messages_id','ASC')->count();
                                            if($chat_array){
                                                foreach($chat_array as $key => $k){
                                                    $senderDetails = User::where('users_id',$k['userId'])->where('status','Active')->first();
                                                    $chat_array[$key]['name']     = $senderDetails->user_name;
                                                    if($senderDetails->profile_picture != '' or $senderDetails->profile_picture != null){
                                                        $chat_array[$key]['profile_pic']     = config('base_urls.profile_images_base_url') . $senderDetails->profile_picture;
                                                    } else{
                                                        $chat_array[$key]['profile_pic']     = '';
                                                    }           
                                                }

                                                $finalDataset = array(
                                                    "chat_length" => $chat_length,
                                                    "unread_messages" => $chat_array,
                                                );
                                                
                                                $response = Helper::success(200, 'success', $finalDataset);
                                            } 
                                            else
                                                  $response = Helper::failed(404, 'error', 'Unupdated chat not found');
                                    } else
                                            $response = Helper::failed(404, 'error', 'No chat found');

                                } else
                                        $response = Helper::failed(404, 'error', 'All fields are needed');

                                break;
                                default:
                                     $response = Helper::failed(404, 'error', 'Invaild Request type passed');
                                    break;
                     }
    } else
        $response = Helper::failed(404, 'error', 'Request type not passed');

    return Helper::custom_response($response);            

}
       



/*** ACCEPT GROUP JOINING REQUEST  ***/
public  function accept_joining_request(Request $req){

    if(isset($req->joiningUserId) && isset($req->groupId) && isset($req->notificationId) ){
                Notification::where('notification_id', $req->notificationId)->update(['status'=>'Responded']);
            GroupMember::where([ ['group_id', $req->groupId], ['users_id', $req->joiningUserId] ])->update(['status' => 'Accepted']);
                //sending push notification 
                $groupDetails = Group::where('group_id', $req->groupId)->first();
                $senderDetails = User::where('users_id',  $groupDetails->users_id)->first();
                $receiverDetails = User::where('users_id', $req->joiningUserId)->first();

                if($senderDetails->users_id != $receiverDetails->users_id){
                    if($receiverDetails->notification_status == 'True'){
                        $notificationResponse =  Helper::sendpushnotification($receiverDetails->one_signal_id, $senderDetails->user_name.' has accepted your group joining request ', 'groupJoinAccept');
                        if($notificationResponse){
                            $notificationData = array(
                                'sender_users_id' => $senderDetails-> users_id,
                                'receiver_users_id' => $receiverDetails-> users_id,
                                'notification_type' => 'GroupJoinAccept',
                                'message' => $senderDetails->user_name.' has accepted your group joining request ',
                                'group_id' => $req->groupId,
                                'datetime' => date('Y-m-d H:i:s'),
                                'status' => 'Unread'
                            );
                            $notificationInsertQuery = Notification::insert($notificationData); 
                        }
                    }
                }

                $response =  Helper::success(200, 'success', 'Joining request accepted successfully');
    } else 
        $response =  Helper::failed(404, 'error', 'All fields are needed');

    return Helper::custom_response($response);

}


/*** REJECT GROUP JOINING REQUEST  ***/
public  function reject_joining_request(Request $req){
    if(isset($req->joiningUserId) && isset($req->groupId) && isset($req->notificationId)){
        Notification::where('notification_id', $req->notificationId)->update(['status'=>'Responded']);

            $joiningQuery = GroupMember::where([ ['group_id', $req->groupId], ['users_id', $req->joiningUserId] ])->update(['status' => 'Rejected']);

                        //sending push notification 
                        $groupDetails = Group::where('group_id', $req->groupId)->first();
                        $senderDetails = User::where('users_id',  $groupDetails->users_id)->first();
                        $receiverDetails = User::where('users_id', $req->joiningUserId)->first();
        
                        if($senderDetails->users_id != $receiverDetails->users_id){
                            if($receiverDetails->notification_status == 'True'){
                                $notificationResponse =  Helper::sendpushnotification($receiverDetails->one_signal_id, $senderDetails->user_name.' has denied your group joining request ', 'groupJoinReject');
                                if($notificationResponse){
                                    $notificationData = array(
                                        'sender_users_id' => $senderDetails-> users_id,
                                        'receiver_users_id' => $receiverDetails-> users_id,
                                        'notification_type' => 'GroupJoinReject',
                                        'message' => $senderDetails->user_name.' has denied your group joining request ',
                                        'group_id' => $req->groupId,
                                        'datetime' => date('Y-m-d H:i:s'),
                                        'status' => 'Unread'
                                    );
                                    $notificationInsertQuery = Notification::insert($notificationData); 
                                }
                            }
                        }

                        
                $response =  Helper::success(200, 'success', 'Joining request rejected');
    } else 
        $response =  Helper::failed(404, 'error', 'All fields are needed');
    return Helper::custom_response($response);
}



/*** ACCEPT GROUP INVITE REQUEST  ***/
public  function accept_invite_request(Request $req){

    if(isset($req->invitedUserId) && isset($req->groupId) && isset($req->notificationId)){
        Notification::where('notification_id', $req->notificationId)->update(['status'=>'Responded']);

            GroupMember::where([ ['group_id', $req->groupId], ['users_id', $req->invitedUserId] ])->update(['status' => 'Accepted']);
                //sending push notification 
                $groupDetails = Group::where('group_id', $req->groupId)->first();
                $senderDetails = User::where('users_id',  $groupDetails->users_id)->first();
                $receiverDetails = User::where('users_id', $req->invitedUserId)->first();

                if($senderDetails->users_id != $receiverDetails->users_id){
                    if($receiverDetails->notification_status == 'True'){
                        $notificationResponse =  Helper::sendpushnotification($receiverDetails->one_signal_id, $senderDetails->user_name.' has accepted your group invitation ', 'groupInviteAccept');
                        if($notificationResponse){
                            $notificationData = array(
                                'sender_users_id' => $senderDetails-> users_id,
                                'receiver_users_id' => $receiverDetails-> users_id,
                                'notification_type' => 'GroupInviteAccept',
                                'message' => $senderDetails->user_name.' has accepted your group invitation ',
                                'group_id' => $req->groupId,
                                'datetime' => date('Y-m-d H:i:s'),
                                'status' => 'Unread'
                            );
                            $notificationInsertQuery = Notification::insert($notificationData); 
                        }
                    }
                }

                $response =  Helper::success(200, 'success', 'Joining request accepted successfully');
    } else 
        $response =  Helper::failed(404, 'error', 'All fields are needed');

    return Helper::custom_response($response);

}




/*** REJECT GROUP INVITE REQUEST -> IN THIS CASE GROUP OWNER SENDED INVITATION WILL BE REJECTED BY INVITED-USER ***/
public  function reject_invite_request(Request $req){
    if(isset($req->invitedUserId) && isset($req->groupId) && isset($req->notificationId)){
        Notification::where('notification_id', $req->notificationId)->update(['status'=>'Responded']);

            $joiningQuery = GroupMember::where([ ['group_id', $req->groupId], ['users_id', $req->invitedUserId] ])->update(['status' => 'Rejected']);

                    //sending push notification 
                    $groupDetails = Group::where('group_id', $req->groupId)->first();
                    $senderDetails = User::where('users_id',  $groupDetails->users_id)->first();
                    $receiverDetails = User::where('users_id', $req->invitedUserId)->first();
    
                    if($senderDetails->users_id != $receiverDetails->users_id){
                        if($receiverDetails->notification_status == 'True'){
                            $notificationResponse =  Helper::sendpushnotification($receiverDetails->one_signal_id, $senderDetails->user_name.' has denied your group invitation ', 'groupInviteReject');
                            if($notificationResponse){
                                $notificationData = array(
                                    'sender_users_id' => $senderDetails-> users_id,
                                    'receiver_users_id' => $receiverDetails-> users_id,
                                    'notification_type' => 'GroupInviteReject',
                                    'message' => $senderDetails->user_name.' has denied your group invitation ',
                                    'group_id' => $req->groupId,
                                    'datetime' => date('Y-m-d H:i:s'),
                                    'status' => 'Unread'
                                );
                                $notificationInsertQuery = Notification::insert($notificationData); 
                            }
                        }
                    }

                $response =  Helper::success(200, 'success', 'Joining request rejected');
    } else 
        $response =  Helper::failed(404, 'error', 'All fields are needed');
    return Helper::custom_response($response);
}


    
     /*** UPLOAD CHAT ATTACHMENT API ***/
     public function upload_chat_attachment(Request $req){
        $response = array();
        $postfix = 'flockdock_attachment';
        $upd_prefix = date('d-m-y');
        $random = rand(1,1000000);

        $file_name = $upd_prefix.$random.$postfix.'.jpeg';
        $chat_attachments_path_url= config('base_urls.relative_path_chat_attachments');

        $uploadedVideo = $req->file('attachment')->move($chat_attachments_path_url,$file_name);
         if($uploadedVideo){
            $response["code"] = 200;
            $response["status"] = "success";
            $response["data"] = $file_name;        
         } else{
            $response["code"] = 500;
            $response["status"] = "error";
            $response["message"] = "Uploading failed";
         }

        return response()
        ->json(array('status' => $response["status"], isset($response["message"]) ? 'message' : 'data' => isset($response["message"]) ? $response["message"] : $response["data"]))
        ->header('Content-Type', 'application/json');
     }
  

     /*** GET SAVED PHRASES  ***/
public  function get_saved_phrases(Request $req){
            $savedPhrases = SavedPhrase::all();
                $response =  Helper::success(200, 'success', $savedPhrases);

    return Helper::custom_response($response);
}




/*** CHAT IMAGES GALLERY  ***/
public  function chat_images_gallery(Request $req){
    $tempDataset = array();
    $mainDataset = array();
    $recentImages = array();
    $lastWeekImages = array();
    $lastMonthImages = array();

    if(isset($req->usersId) && isset($req->senderId)){
        $chatImages = ChatMessage::where([ ['sender_id', $req->senderId], ['message_type', 'attachment'] ])
            ->where([ ['receiver_id', $req->usersId], ['message_type', 'attachment'] ])->get();

                 if(sizeof($chatImages) > 0){
                        foreach($chatImages as $k){
                            $senderDetails = User::where('users_id', $k->sender_id)->first();
                            $customObj = new stdClass();
                            $customObj->user_name = $senderDetails->user_name;
                            $customObj->created_at = $k->created_at;
                            $customObj->picture = config('base_urls.chat_attachments_base_url').json_decode($k->message);
                            $customObj->formatted_datetime = date('d F, h:i A', strtotime($k->created_at));
                            $customObj->created_at = $k->created_at;

                            array_push($tempDataset, $customObj);
                        }


                        foreach($tempDataset as $k){
                            $createdAt = new DateTime($k->created_at);
                            $currentDateTime = new DateTime();
    
                            $diff = $createdAt->diff($currentDateTime);
                            if($diff->d < 7 && $diff->m < 1)
                                array_push($recentImages, $k);
                            else if($diff->d > 7 && $diff->m < 1)
                                array_push($lastWeekImages, $k);
                            else    
                                array_push($lastMonthImages, $k);
                            
                        }
                        
                        $mainDataset['recent_images'] = $recentImages;
                        $mainDataset['last_week_images'] = $lastWeekImages;
                        $mainDataset['last_month_images'] = $lastMonthImages;

                    $response =  Helper::success(200, 'success', $mainDataset);

                 }else
                      $response =  Helper::failed(404, 'error', 'No Chat Images found');

          

    } else 
        $response =  Helper::failed(404, 'error', 'All fields are needed');
    return Helper::custom_response($response);
}

/* --------------- SEARCH MESSAGE --------------- */
public function search_message(Request $req){
    if (isset($req->requestType) && isset($req->userId) && isset($req->otherUserId) && isset($req->message)) {
        if ($req->requestType == 'searchMessage') {
            // get data
            $data = ChatMessage::where([ ['message_type', '!=', 'attachment'],['message_type', '!=', 'location'],['message', 'Like', '%'.$req->message.'%'],['sender_id', $req->userId], ['receiver_id', $req->otherUserId]])
                                ->orWhere([ ['message_type', '!=', 'attachment'],['message_type', '!=', 'location'],['message', 'Like', '%'.$req->message.'%'],['sender_id', $req->otherUserId], ['receiver_id', $req->userId]])->get();
            // get data
            
            if (count($data) > 0) {
                $response['code'] = 200;
                $response['status'] = 'success';
                $response['data'] = $data;
            } else{
                $response['code'] = 404;
                $response['status'] = 'error';
                $response['message'] = 'No message found.';
            }
        } else{
            $response['code'] = 404;
            $response['status'] = 'error';
            $response['message'] = 'Request type is not correct.';
        }
    } else{
        $response['code'] = 404;
        $response['status'] = 'error';
        $response['message'] = 'All fields are required.';
    }

    return response()
    ->json(['status' => $response['status'], isset($response['message']) ? 'message' : 'data' => isset($response['message']) ? $response['message'] : $response['data']])
    ->header('Content-Type', 'application/json');
}
/* --------------- SEARCH MESSAGE --------------- */
  
}





