<?php
namespace App\Http\Controllers;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use App\Http\Controllers\Controller;
use Mail;
use Carbon\Carbon;
use Illuminate\Support\Facades\Hash;
use File;
use DB;
use \stdClass;
use Stripe\Stripe;
use Stripe\Charge;
use Stripe\Customer;
use Stripe\StripeClient;
use DateTime;
use Illuminate\Support\Arr;
use App\Models\User;
use App\Models\Ethnicity;
use App\Models\BodyType;
use App\Models\Position;
use App\Models\Relationship;
use App\Models\Seeking;
use App\Models\UserSeeking;
use App\Models\Tribe;
use App\Models\UserPicture;
use App\Models\BlockUser;
use App\Models\UserTribe;
use App\Models\PrivateGalleryAccess;
use App\Helpers\Helper;
use App\Models\Notification;





class ProfileSettingsController extends Controller{
    public $successStatus = 200;    
    public $errorStatus = 401;

    /*** SET NOTIFICATION SETTINGS ***/
    public  function set_notification_settings(Request $req)
    {
        $response = array();
        
        if (isset($req->usersId) &&  isset($req->notificationSetting)) {
                User::where('users_id', $req->usersId)->update(['notification_status' =>  $req->notificationSetting]);
                $response["code"] = 200;
                $response["status"] = "success";
                $response["data"] = "Setting updated successfully";
        } else {
            $response["code"] = 404;
            $response["status"] = "error";
            $response["message"] = "All fields are needed.";
        }

       
        return response()
                    ->json(array('status' => $response["status"], isset($response["message"]) ? 'message' : 'data' => isset($response["message"]) ? $response["message"] : $response["data"]))
                    ->header('Content-Type', 'application/json');

    }
    public function Destroy(Request $req)
    {
        
        if (isset($req->notification_id))
        {
                $id=$req->notification_id;
                $Notification = Notification::find($id);
            if($Notification)
            {
                $Notification->delete();
                return response()->json(['message'=>'Notification Deleted Successfully'],200);
            }
            else
            {
                return response()->json(['message'=>'Notification Not Found'],404);
            }
        }
        else
        {
            return response()->json(['message'=>'Notification ID Not Found'],404);

        }
        
    }


public  function get_predefined_profile_details(Request $req){
    $response = array();
    $mainDataset = array();

     $ethnicities = Ethnicity::where('status', 'Active')->get();
     $body_types = BodyType::where('status', 'Active')->get();
     $positions = Position::where('status', 'Active')->get();
     $relationships = Relationship::where('status', 'Active')->get();
     $seekings = Seeking::where('status', 'Active')->get();
     $tribes = Tribe::where('status', 'Active')->get();


     $mainDataset['ethnicities'] = $ethnicities;
     $mainDataset['body_types'] = $body_types;
     $mainDataset['positions'] = $positions;
     $mainDataset['relationships'] = $relationships;
     $mainDataset['seekings'] = $seekings;
     $mainDataset['tribes'] = $tribes;

     $response["code"] = 200;
     $response["status"] = "success";
     $response["data"] = $mainDataset;


return response()
         ->json(array('status' => $response["status"], isset($response["message"]) ? 'message' : 'data' => isset($response["message"]) ? $response["message"] : $response["data"]))
         ->header('Content-Type', 'application/json');

}



public  function get_user_profile_details(Request $req){
    // $response = array();
    
   if(isset($req->usersId)){
       $userData = User::where('users_id',$req->usersId)->first();

       //notfication status


       if($userData->profile_picture != '' || $userData->profile_picture != null)
         $userData->profile_picture     = config('base_urls.profile_images_base_url') . $userData->profile_picture;
         else
            $userData->profile_picture     = '';


        $userData->user_tribes = UserTribe::where('users_id', $req->usersId)->pluck('tribe_id');
        $userData->user_seekings = UserSeeking::where('users_id', $req->usersId)->pluck('seeking_id');

        $userData = $userData->makeHidden(['password','confirm_password','verify_code']);

        $response["code"] = 200;
        $response["status"] = "success";
        $response["data"] = $userData;

   }else{
        $response["code"] = 404;
        $response["status"] = "error";
    $response["message"] = "All fields are needed.";
   }


return response()
         ->json(array('status' => $response["status"], isset($response["message"]) ? 'message' : 'data' => isset($response["message"]) ? $response["message"] : $response["data"]))
         ->header('Content-Type', 'application/json');

}




public  function delete_user_profile_picture(Request $req){
    $response = array();
    
   if(isset($req->usersId)){
       $profileUpdateQuery = User::where('users_id',$req->usersId)->update(['profile_picture' => NULL]);

       $response["code"] = 200;
        $response["status"] = "success";
        $response["data"] = 'Profile picture deleted successfully';

   }else{
        $response["code"] = 404;
        $response["status"] = "error";
    $response["message"] = "All fields are needed.";
   }


return response()
         ->json(array('status' => $response["status"], isset($response["message"]) ? 'message' : 'data' => isset($response["message"]) ? $response["message"] : $response["data"]))
         ->header('Content-Type', 'application/json');

}



public function upload_user_profile_picture(Request $req){
    $response = array();
    $postfix = '_flockdock_user_profile';
    $upd_prefix = date('d-m-y');
    $random = rand(1,1000000);

    $file_name = $upd_prefix.$random.$postfix.'.jpeg';
    $attachment_path_url= config('base_urls.relative_path_profile_images');

    $uploadedAttachment = $req->file('user_profile_picture')->move($attachment_path_url,$file_name);
    if($uploadedAttachment){
        
        $response["code"] = 200;
        $response["status"] = "success";
        $response["data"] = $file_name;        

    } else{
        $response["code"] = 500;
        $response["status"] = "error";
        $response["message"] = "Uploading failed";
    }

    return response()
    ->json(array('status' => $response["status"], isset($response["message"]) ? 'message' : 'data' => isset($response["message"]) ? $response["message"] : $response["data"]))
    ->header('Content-Type', 'application/json');
}






public function upload_user_profile_picture_details(Request $req){
    $response = array();
    

   if(isset($req->usersId) && isset($req->profilePicture)){
       $updateQuery  = User::where('users_id', $req->usersId)->update(['profile_picture' => $req->profilePicture]);

        $response["code"] = 200;
        $response["status"] = "success";
        $response["data"] = config('base_urls.profile_images_base_url').$req->profilePicture;

   }else{
        $response["code"] = 404;
        $response["status"] = "error";
    $response["message"] = "All fields are needed.";
   }


return response()
         ->json(array('status' => $response["status"], isset($response["message"]) ? 'message' : 'data' => isset($response["message"]) ? $response["message"] : $response["data"]))
         ->header('Content-Type', 'application/json');

}



public  function get_user_pictures(Request $req){
    $response = array();
    

   if(isset($req->usersId)){
            $mainDataset = new stdClass();
       $visiblePictures = UserPicture::userTypePictures($req->usersId, 'Visible')->get();
          foreach($visiblePictures as $k){
              $k->picture = config('base_urls.user_picture_base_url').$k->picture;
          }
       $privatePictures = UserPicture::userTypePictures($req->usersId, 'Private')->get();
            foreach($privatePictures as $k){
                        $k->picture = config('base_urls.user_picture_base_url').$k->picture;
                    }

            $mainDataset->visible_pictures = $visiblePictures;
            $mainDataset->private_pictures = $privatePictures;

        $response["code"] = 200;
        $response["status"] = "success";
        $response["data"] = $mainDataset;

   }else{
        $response["code"] = 404;
        $response["status"] = "error";
    $response["message"] = "All fields are needed.";
   }


return response()
         ->json(array('status' => $response["status"], isset($response["message"]) ? 'message' : 'data' => isset($response["message"]) ? $response["message"] : $response["data"]))
         ->header('Content-Type', 'application/json');

}



public function upload_user_picture(Request $req){
    $response = array();
    $postfix = '_flockdock_user_picture';
    $upd_prefix = date('d-m-y');
    $random = rand(1,1000000);

    $file_name = $upd_prefix.$random.$postfix.'.jpeg';
    $attachment_path_url= config('base_urls.relative_path_user_picture');

    $uploadedAttachment = $req->file('user_picture')->move($attachment_path_url,$file_name);
    if($uploadedAttachment){
        $response["code"] = 200;
        $response["status"] = "success";
        $response["data"] = $file_name;        

    } else{
        $response["code"] = 500;
        $response["status"] = "error";
        $response["message"] = "Uploading failed";
    }

    return response()
    ->json(array('status' => $response["status"], isset($response["message"]) ? 'message' : 'data' => isset($response["message"]) ? $response["message"] : $response["data"]))
    ->header('Content-Type', 'application/json');
}



public  function upload_user_picture_details(Request $req){
    $response = array();
    

   if(isset($req->usersId) && isset($req->picture) && isset($req->pictureType)){
       $pictureData = array(
           'users_id' => $req->usersId,
           'picture' => $req->picture,
           'picture_type' => $req->pictureType,
           'created_at' => date('Y-m-d H:i:s'),
           'status' => 'Active',
       );
       $insertQuery = UserPicture::insert($pictureData);


        $response["code"] = 200;
        $response["status"] = "success";
        $response["data"] = 'User picture added successfully';

   }else{
        $response["code"] = 404;
        $response["status"] = "error";
    $response["message"] = "All fields are needed.";
   }


return response()
         ->json(array('status' => $response["status"], isset($response["message"]) ? 'message' : 'data' => isset($response["message"]) ? $response["message"] : $response["data"]))
         ->header('Content-Type', 'application/json');

}





public  function delete_user_picture(Request $req){
    $response = array();
    
   if(isset($req->userPictureId)){
       $deletePictureQuery = UserPicture::where('user_picture_id',$req->userPictureId)->delete();

        $response["code"] = 200;
        $response["status"] = "success";
        $response["data"] = 'User picture deleted successfully';

   }else{
        $response["code"] = 404;
        $response["status"] = "error";
    $response["message"] = "All fields are needed.";
   }


return response()
         ->json(array('status' => $response["status"], isset($response["message"]) ? 'message' : 'data' => isset($response["message"]) ? $response["message"] : $response["data"]))
         ->header('Content-Type', 'application/json');

}



public  function update_profile_details(Request $req){
    $response = array();
    
   if(isset($req->usersId) && isset($req->gender) && isset($req->showAge) && isset($req->ethnicityId)
        && isset($req->bodyTypeId) && isset($req->positionId) && isset($req->relationshipId)  && isset($req->hivStatus)
         && isset($req->covidVaccine)){

       $updateData = array(
           'gender' => $req->gender,
           'birthday' => (isset($req->birthday) ? date('Y-m-d', strtotime($req->birthday)) : null),
           'show_age' => $req->showAge,
           'height' => (isset($req->height) ? $req->height : null),
           'weight' => (isset($req->weight) ? $req->weight : null),
           'ethnicity_id' => $req->ethnicityId,
           'body_type_id' => $req->bodyTypeId,
           'position_id' => $req->positionId,
           'relationship_id' => $req->relationshipId,
           'hiv_status' => $req->hivStatus,
           'description' => (isset($req->description) ? $req->description : null),
           'date_of_last_test' => (isset($req->dateOfLastTest) ? date('Y-m-d', strtotime($req->dateOfLastTest)) : null),
           'covid_vaccine' => $req->covidVaccine,
           'facebook_link' => (isset($req->facebookLink) ? $req->facebookLink : null),
           'instagram_link' => (isset($req->instagramLink) ? $req->instagramLink : null),
           'twitter_link' => (isset($req->twitterLink) ? $req->twitterLink : null),
       );
      
         User::where('users_id', $req->usersId)->update($updateData);

       if((isset($req->tribeIds))) {
           UserTribe::where('users_id', $req->usersId)->delete();
           foreach ($req->tribeIds as $k) {
               UserTribe::insert([
                   'users_id' => $req->usersId,
                   'tribe_id' => $k
               ]);
           }
       }

       if((isset($req->seekingIds))) {
           UserSeeking::where('users_id', $req->usersId)->delete();
           foreach ($req->seekingIds as $k) {
               UserSeeking::insert([
                   'users_id' => $req->usersId,
                   'seeking_id' => $k
               ]);
           }
       }

        $response["code"] = 200;
        $response["status"] = "success";
        $response["data"] = 'profile data updated successfully';

   }else{
        $response["code"] = 404;
        $response["status"] = "error";
    $response["message"] = "All fields are needed.";
   }


return response()
         ->json(array(
             'status' => $response["status"], 
             isset($response["message"]) ? 'message' : 'data' => isset($response["message"]) ? $response["message"] : $response["data"]))
         ->header('Content-Type', 'application/json');

}



/*** USER PROFILE SETTINGS ***/
public function get_user_profile_settings(Request $req){
    $response = array();

    if (isset($req->usersId)) {
        $profileSettingDetails = new stdClass();
        $userDetails = User::where('users_id', $req->usersId)->first();

        $profileSettingDetails->users_id = $req->usersId; 
        $profileSettingDetails->user_name = $userDetails->user_name; 
        $profileSettingDetails->email = $userDetails->email; 
        $profileSettingDetails->blocked_users_count = BlockUser::where('blocked_by_user_id', $req->usersId)->count(); 
        $profileSettingDetails->unit_system = $userDetails->unit_system; 
            if($userDetails->notification_status == 'True')
                $profileSettingDetails->notification_status = 'Yes'; 
            else
                $profileSettingDetails->notification_status = 'No'; 
                

        $response["code"] = 200;
        $response["status"] = "success";
        $response["data"] = $profileSettingDetails;


    } else{
        $response["code"] = 404;
        $response["status"] = "error";
        $response["message"] = "All fields are needed"; 
    }
    return response()
    ->json(array('status' => $response["status"], isset($response["message"]) ? 'message' : 'data' => isset($response["message"]) ? $response["message"] : $response["data"]))
    ->header('Content-Type', 'application/json');
}




 /*** BLOCK USER API ***/
 public function block_user(Request $req){
    $response = array();
    
    if (isset($req->blockedByUserId) && isset($req->blockedUserId)) {
         
            $checkAlreadyBlocked = BlockUser::where([ ['blocked_user_id' , $req->blockedUserId], ['blocked_by_user_id', $req->blockedByUserId] ])->count();
            if($checkAlreadyBlocked == 0){
                $blockData = array(
                    'blocked_user_id' => $req->blockedUserId,
                    'blocked_by_user_id' => $req->blockedByUserId
                );

                $insertQuery = BlockUser::insert($blockData);
                if($insertQuery){
                    $response["code"] = 200;
                    $response["status"] = "success";
                    $response["data"] = "Blocked Successfully";
                }
            }else{
                $response["code"] = 404;
                $response["status"] = "error";
                $response["message"] = "Already Blocked";
            }
            
    } else{
        $response["code"] = 404;
        $response["status"] = "error";
        $response["message"] = "All fields are needed"; 
    }
    return response()
    ->json(array('status' => $response["status"], isset($response["message"]) ? 'message' : 'data' => isset($response["message"]) ? $response["message"] : $response["data"]))
    ->header('Content-Type', 'application/json');
}



/*** BLOCK LIST API ***/
public function get_block_list(Request $req){
    $response = array();
    if (isset($req->blockByUserId) ) {
            $blockList = BlockUser::where('blocked_by_user_id', $req->blockByUserId)->get();
            
            foreach($blockList as $k){
                $blockedUserDetails = User::where('users_id', $k->blocked_user_id)->first();
                $k->user_name = $blockedUserDetails->user_name;

                if($blockedUserDetails->profile_picture != '' || $blockedUserDetails->profile_picture != null)
                     $k->profile_picture  = config('base_urls.profile_images_base_url') . $blockedUserDetails->profile_picture;
                else
                   $k->profile_picture   = '';

                   //for online status
                   if($blockedUserDetails->online == 'Yes')
                       $k->is_online = true;
                   else
                       $k->is_online = false;
            }

                     $response["code"] = 200;
                    $response["status"] = "success";
                    $response["data"] = $blockList;
            
    } else{
        $response["code"] = 404;
        $response["status"] = "error";
        $response["message"] = "All fields are needed"; 
    }
    return response()
    ->json(array('status' => $response["status"], isset($response["message"]) ? 'message' : 'data' => isset($response["message"]) ? $response["message"] : $response["data"]))
    ->header('Content-Type', 'application/json');
}


 /*** UNBLOCK USER API ***/
 public function unblock_user(Request $req){
    $response = array();
    
    if (isset($req->blockedByUserId) && isset($req->blockedUserId)) {
         
            $checkAlreadyBlocked = BlockUser::where([ ['blocked_user_id' , $req->blockedUserId], ['blocked_by_user_id', $req->blockedByUserId] ])->count();
            if($checkAlreadyBlocked > 0){
                BlockUser::where([ ['blocked_user_id' , $req->blockedUserId], ['blocked_by_user_id', $req->blockedByUserId] ])->delete();
                    $response["code"] = 200;
                    $response["status"] = "success";
                    $response["data"] = "Unblocked Successfully";
            }else{
                $response["code"] = 404;
                $response["status"] = "error";
                $response["message"] = "Blocked user not found";
            }
            
    } else{
        $response["code"] = 404;
        $response["status"] = "error";
        $response["message"] = "All fields are needed"; 
    }
    return response()
    ->json(array('status' => $response["status"], isset($response["message"]) ? 'message' : 'data' => isset($response["message"]) ? $response["message"] : $response["data"]))
    ->header('Content-Type', 'application/json');
}



 /*** UPDATE UNIT SYSTEM PREFERENCE API ***/
 public function update_unit_system_preference(Request $req){
    $response = array();
    if (isset($req->usersId) && isset($req->unitSystem)) {
         
                    User::where('users_id' , $req->usersId)->update(['unit_system' => $req->unitSystem]);
                    $response["code"] = 200;
                    $response["status"] = "success";
                    $response["data"] = "Updated Successfully";
    
            
    } else{
        $response["code"] = 404;
        $response["status"] = "error";
        $response["message"] = "All fields are needed"; 
    }
    return response()
    ->json(array('status' => $response["status"], isset($response["message"]) ? 'message' : 'data' => isset($response["message"]) ? $response["message"] : $response["data"]))
    ->header('Content-Type', 'application/json');
}


 /*** UPDATE EMAIL USERNAME PASSWORD ***/
 public function update_credentials(Request $req){
    $response = array();

    if (isset($req->usersId)) {

        if(isset($req->userName)){
            //Check Already used username
            $usernameAlreadyUsed = User::where('user_name', $req->userName)->count();
              if($usernameAlreadyUsed == 0){
                     User::where('users_id', $req->usersId)->update(['user_name' => $req->userName]);
                     $response["code"] = 200;
                    $response["status"] = "success";
                    $response["data"] = "Credentials Updated Successfully"; 
                }else{
                    $response["code"] = 404;
                    $response["status"] = "error";
                    $response["message"] = "Username already used, Try something unique"; 
                }
          }
          
        if(isset($req->userEmail)){
            //Check Already used username
            $useremailAlreadyUsed = User::where('email', $req->userEmail)->count();
            if($useremailAlreadyUsed == 0){
                     User::where('users_id', $req->usersId)->update(['email' => $req->userEmail]);
                     $response["code"] = 200;
                    $response["status"] = "success";
                    $response["data"] = "Credentials Updated Successfully"; 
                }else{
                    $response["code"] = 404;
                    $response["status"] = "error";
                    $response["message"] = "Email already used, Try something unique"; 
                }
          }

          if(isset($req->password)){
                User::where('users_id', $req->usersId)->update(['password' => password_hash($req->userPassword, PASSWORD_DEFAULT)]);
                $response["code"] = 200;
                $response["status"] = "success";
                $response["data"] = "Credentials Updated Successfully"; 
          }
            
    } else{
        $response["code"] = 404;
        $response["status"] = "error";
        $response["message"] = "All fields are needed"; 
    }
    return response()
    ->json(array('status' => $response["status"], isset($response["message"]) ? 'message' : 'data' => isset($response["message"]) ? $response["message"] : $response["data"]))
    ->header('Content-Type', 'application/json');
}


/*** REQUEST PRIVATE GALLERY ***/
public function request_private_gallery(Request $req){
    $response = array();

    if (isset($req->requestedByUserId) && isset($req->requestedToUserId) ) {
        //Check Already used username
        $checkGalleryAccess = PrivateGalleryAccess::checkPrivateGalleryAccess($req->requestedToUserId, $req->requestedByUserId);
        if($checkGalleryAccess){ 
            //Have might requested before 
                    if($checkGalleryAccess->status == 'Pending'){
                        $response["code"] = 200;
                        $response["status"] = "success";
                        $response["data"] = "Already Requested";
                    }
                    elseif($checkGalleryAccess->status == 'Allowed'){
                        $response["code"] = 200;
                        $response["status"] = "success";
                        $response["data"] = "Already Allowed";
                    }else{
                        $response["code"] = 200;
                        $response["status"] = "success";
                        $response["data"] = "Already Denied";
                    }
        
        }else{
            //new Request for gallery access
            privateGalleryAccess::insert([
                        'requested_to_user_id' => $req->requestedToUserId,
                        'requested_by_user_id' => $req->requestedByUserId,
                        'created_at'=> date('Y-m-d H:i:s'),
                        'status' => 'Pending'
                    ]);

                    $response["code"] = 200;
                    $response["status"] = "success";
                    $response["data"] = "Requested to user";
        }
            
    } else{
        $response["code"] = 404;
        $response["status"] = "error";
        $response["message"] = "All fields are needed"; 
    }
    return response()
    ->json(array('status' => $response["status"], isset($response["message"]) ? 'message' : 'data' => isset($response["message"]) ? $response["message"] : $response["data"]))
    ->header('Content-Type', 'application/json');
}


/*** REQUEST PRIVATE GALLERY ***/
public function get_gallery_permission_members(Request $req){
    $response = array();

    $mainDataset = array();

    if (isset($req->usersId) && isset($req->permissionType) ) {
            if($req->permissionType == 'Allowed')
                 $members = PrivateGalleryAccess::where([ ['requested_to_user_id', $req->usersId], ['status', 'Allowed'] ])->get();
            else    
                $members = PrivateGalleryAccess::where([ ['requested_to_user_id', $req->usersId], ['status', 'Denied'] ])->get();

        if(sizeof($members) > 0){
            foreach($members as $k){
                $memberDetails = new stdClass();
                $userDetails = User::where('users_id', $k->requested_by_user_id)->first();

                $memberDetails->users_id = $k->requested_by_user_id;
                $memberDetails->user_name = $userDetails->user_name;
               
                if($userDetails->profile_picture != '' || $userDetails->profile_picture != null)
                     $memberDetails->profile_picture     = config('base_urls.profile_images_base_url') . $userDetails->profile_picture;
                else
                    $memberDetails->profile_picture     = '';
                 $memberDetails->member_status = $k->status;

                 array_push($mainDataset, $memberDetails);
            }

            $response["code"] = 200;
            $response["status"] = "success";
            $response["data"] = $mainDataset;
            
        }else{
            $response["code"] = 404;
            $response["status"] = "error";
            $response["message"] = "No members found"; 
        }
                   
            
    } else{
        $response["code"] = 404;
        $response["status"] = "error";
        $response["message"] = "All fields are needed"; 
    }
    return response()
    ->json(array('status' => $response["status"], isset($response["message"]) ? 'message' : 'data' => isset($response["message"]) ? $response["message"] : $response["data"]))
    ->header('Content-Type', 'application/json');
}



/*** ALLOW PERMISSION ***/
public function allow_permission_to_users(Request $req){
    $response = array();

    if (isset($req->usersId) && isset($req->allowingUserIds) ) {
           foreach($req->allowingUserIds as $k){
               PrivateGalleryAccess::where([ 
                                        ['requested_to_user_id', $req->usersId], 
                                        ['requested_by_user_id', $k] 
                                    ])->update(['status' => 'Allowed']);
            
            $response["code"] = 200;
            $response["status"] = "success";
            $response["data"] = 'Allowed Successfully';
           }
            
    } else{
        $response["code"] = 404;
        $response["status"] = "error";
        $response["message"] = "All fields are needed"; 
    }
    return response()
    ->json(array('status' => $response["status"], isset($response["message"]) ? 'message' : 'data' => isset($response["message"]) ? $response["message"] : $response["data"]))
    ->header('Content-Type', 'application/json');
}


/*** DENY PERMISSION ***/
public function deny_permission_to_users(Request $req){
    $response = array();

    if (isset($req->usersId) && isset($req->denyingUserIds) ) {
           foreach($req->denyingUserIds as $k){
               PrivateGalleryAccess::where([ 
                                        ['requested_to_user_id', $req->usersId], 
                                        ['requested_by_user_id', $k] 
                                    ])->update(['status' => 'Denied']);
            
            $response["code"] = 200;
            $response["status"] = "success";
            $response["data"] = 'Denied Successfully';
           }
            
    } else{
        $response["code"] = 404;
        $response["status"] = "error";
        $response["message"] = "All fields are needed"; 
    }
    return response()
    ->json(array('status' => $response["status"], isset($response["message"]) ? 'message' : 'data' => isset($response["message"]) ? $response["message"] : $response["data"]))
    ->header('Content-Type', 'application/json');
}

    public function update_notification_status(Request $request)
    {
        $response = Helper::success(200, 'success', null);

        //Validation
        if (!isset($request->userId) || !isset($request->notificationStatus)) {
            $response = Helper::failed(404, 'Denied', "userId and notificationStatus parameters are required!");
            return Helper::custom_response($response);
        }

        $user_id = $request->userId;
        $user_status = $request->notificationStatus == 'Yes' ? 'True' : 'False';

        $user = User::find($user_id);
        if ($user) {
            $user->notification_status = $user_status;
            if ($user->save()) {
                $response = Helper::success(200, 'Success', "Notification Status Updated Successfully!");
            } else {
                $response = Helper::failed(404, 'Denied', "Something is Wrong, Notification Status Not Updated!");
            }
        } else {
            $response = Helper::failed(404, 'Denied', "User not found against User Id " . $user_id);
        }

        return Helper::custom_response($response);
    }



}