<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Http;

class ReferralController extends Controller
{
    private $apiBaseUrl;

    public function __construct()
    {
        $this->apiBaseUrl = config('services.referral_api.base_url');
    }

    public function validateReferralCode(Request $request)
    {
        $request->validate([
            'referral_code' => 'required|string'
        ]);

        $response = Http::post($this->apiBaseUrl . '/VerifyReferral.php', [
            'referral_code' => $request->referral_code
        ]);

        return $response->json();
    }

    public function recordReferral(Request $request)
    {
        $request->validate([
            'referral_id' => 'required|integer',
            'referred_sipUserName' => 'required|string'
        ]);

        $response = Http::post($this->apiBaseUrl . '/RecordReferral.php', [
            'referral_id' => $request->referral_id,
            'referred_sipUserName' => $request->referred_sipUserName
        ]);

        return $response->json();
    }

    public function generatePromoCode(Request $request)
    {
        $request->validate([
            'sipUserName' => 'required|string'
        ]);

        $response = Http::get($this->apiBaseUrl . "/generate_promo_code.php/{$request->sipUserName}");

        return $response->json();
    }

    public function getPromoCode(Request $request)
    {
        $request->validate([
            'sipUsername' => 'required|string'
        ]);

        $response = Http::get($this->apiBaseUrl . '/get_Promo_code.php', [
            'sipUsername' => $request->sipUsername
        ]);

        return $response->json();
    }

    public function getReferralSummary(Request $request)
    {
        $request->validate([
            'sipDeviceUsername' => 'required|string'
        ]);

        $response = Http::get($this->apiBaseUrl . "/get_referral_summary.php/{$request->sipDeviceUsername}");

        return $response->json();
    }

    public function getReferrals(Request $request)
    {
        $request->validate([
            'sipDeviceUsername' => 'required|string'
        ]);

        $response = Http::get($this->apiBaseUrl . "/get_referrals.php/{$request->sipDeviceUsername}");

        return $response->json();
    }
}
