<?php

namespace App\Http\Controllers;

use App\Helpers\ApiService;
use App\Helpers\CustomerLogic;
use App\Models\CustomerTran;
use App\Models\StripeCustomer;
use App\Models\SubscriptionProduct;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Cookie;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Str;
use Stripe\Exception\SignatureVerificationException;
use Stripe\PaymentIntent;
use Stripe\Price;
use Stripe\Product;
use Stripe\Stripe;
use Stripe\Subscription;
use Stripe\Subscription as StripeSubscription;
use Stripe\Webhook;
use function Symfony\Component\Console\Style\comment;

class StripeController extends Controller
{

    protected $apiService;

    public function __construct(APIService $apiService)
    {
        $this->apiService = $apiService;
        $date = gmdate("Y_m_d");
        $this->fp= fopen("/var/log/stripe/stripe_".$date.".log","a+");
    }

    public function __destruct()
    {
        if ($this->fp) {
            fclose($this->fp);
        }
    }

    public function stripe_topup(Request $request)
    {
        fwrite($this->fp,"[stripe_topup] function called".PHP_EOL);
        $balance= $request->balance;
        if ($balance < 1){
            return redirect()->back();
        }
        $domain = request()->getHost();
        $url = url()->route('stripe_topup_charge');
        $urlWithoutDomain = str_replace(url('/'), '', $url);
        $main_url = $domain.$urlWithoutDomain;
        return view('stripe.stripe_topup', compact('balance', 'main_url'));
    }

    public function stripe_topup_charge(Request $request)
    {
        fwrite($this->fp,"[stripe_topup_charge] function called".PHP_EOL);
        $customer_info = Cookie::get('customer_info');
        $customer_info = json_decode($customer_info, true);

        $response = $this->apiService->callAPI('POST', 'api/global_api/', [
            "action" => "get_country_list",
            "device_id" =>  "",
        ]);

        $unique_id = Str::random(6) . '-' . rand(1, 1000).'-'.uniqid();
        session()->put('unique_id', $unique_id);

        $country_list = $response['data'];

        $country = collect($country_list)->where('id', $customer_info['country_id'])->first()['iso3'];

        $amount = $request->balance;

        Stripe::setApiKey(env('STRIPE_SECRET'));
        header('Content-Type: application/json');

        $currency_code = 'USD';
        $currencies_not_supported_cents = ['BIF', 'CLP', 'DJF', 'GNF', 'JPY', 'KMF', 'KRW', 'MGA', 'PYG', 'RWF', 'UGX', 'VND', 'VUV', 'XAF', 'XOF', 'XPF'];

        $base_url = url('/');

        $check_customer = StripeCustomer::where('customer_id', $customer_info['id'])->first();
        if (!empty($check_customer)){
            $customer = \Stripe\Customer::update($check_customer['stripe_customer_id'], [
                'email' => $customer_info['email'],
                'address' => [
                    'city' => $customer_info['city'],
                    'country' => $country, // Pass the customer's country here
                    'line1' =>$customer_info['address_1'],
                    'line2' =>$customer_info['address_2'],
                    'postal_code' =>$customer_info['postal_code'],
                ],
            ]);

        }else{
            $customer = \Stripe\Customer::create([
                'email' => $customer_info['email'],
                'address' => [
                    'city' => $customer_info['city'],
                    'country' => $country, // Pass the customer's country here
                    'line1' =>$customer_info['address_1'],
                    'line2' =>$customer_info['address_2'],
                    'postal_code' =>$customer_info['postal_code'],
                ],
            ]);

            $stripe_customer = new StripeCustomer();
            $stripe_customer->customer_id = $customer_info['id'];
            $stripe_customer->stripe_customer_id = $customer->id;
            $stripe_customer->save();
        }

        $checkout_session = \Stripe\Checkout\Session::create([
            'payment_method_types' => ['card'],
            'line_items' => [[
                'price_data' => [
                    'currency' => $currency_code,
                    'unit_amount' => in_array($currency_code, $currencies_not_supported_cents) ? (int)$amount : ($amount * 100),
                    'product_data' => [
                        'name' => 'alotel',
                        'images' => ['test.png'],
                    ],
                ],
                'quantity' => 1,
            ]],
            'customer' => $customer->id,
            'mode' => 'payment',
            'success_url' => $base_url.'/stripe-totpup-success?session_id={CHECKOUT_SESSION_ID}',
            'cancel_url' => route('dashboard'),
            'metadata' => [
                'order_id' => $unique_id,
                'amount' => $amount
            ],
        ]);

        return response()->json(['id' => $checkout_session->id]);
    }

    public function stripe_topup_success(Request $request)
    {
        fwrite($this->fp,"[stripe_topup_success] function called".PHP_EOL);
        if(is_array($request)){
                fwrite($this->fp,"[stripe_topup_success] Request json ".json_encode($request).PHP_EOL);
        }
        Stripe::setApiKey(env('STRIPE_SECRET'));
        $session = \Stripe\Checkout\Session::retrieve($request->get('session_id'));
        if (session('unique_id') != $session->metadata->order_id){
            return redirect()->route('add_credit');
        }

        $tran_id = $session->payment_intent;
        $check = CustomerTran::where('tran_id', $tran_id)->count();
        if ($check < 0){
            notify()->error('Something went to wrong',"Success");
            return redirect()->back();
        }

        $customer_info = Cookie::get('customer_info');
        $customer_info = json_decode($customer_info, true);

        $customer_tran = new CustomerTran();
        $customer_tran->order_id = $session->metadata->order_id;
        $customer_tran->customer_id = $customer_info['id'];
        $customer_tran->stripe_customer_id = $session->customer;
        $customer_tran->tran_id = $tran_id;
        $customer_tran->save();
        $amount = $session->metadata->amount;
        $response = CustomerLogic::add_balance($this->apiService, $customer_info, $amount);
        fwrite($this->fp,"[stripe_topup_success] API Response1 ".$response.PHP_EOL);
        if ($response['response_code'] == 400){
            return redirect()->route('logout');
        }
        session()->forget('unique_id');
        notify()->success($response['success'],"Success");
        return redirect()->route('dashboard');
    }

    public function stripe_payment($product_id)
    {
        fwrite($this->fp,"Stripe_payment function called".PHP_EOL);
        $domain = request()->getHost();
        $url = url()->route('stripe_payment_charge');
        $urlWithoutDomain = str_replace(url('/'), '', $url);
        $main_url = $domain.$urlWithoutDomain;
        return view('stripe.stripe_subscription', compact('product_id', 'main_url'));
    }

    public function stripe_payment_charge(Request $request)
    {
        fwrite($this->fp,"Stripe_payment_charge function called".PHP_EOL);
        $product_id = Crypt::decrypt($request->product_id);
        $product_res = $this->apiService->callAPI('GET', 'product-list.php', []);
        $product = collect($product_res)->where('id', $product_id)
            ->first();

        if (empty($product)){
            notify()->error('Product not found', 'Error');
        }

        $customer_info = Cookie::get('customer_info');
        $customer_info = json_decode($customer_info, true);

        $response = $this->apiService->callAPI('POST', 'api/global_api/', [
            "action" => "get_country_list",
            "device_id" =>  "",
        ]);

        $unique_id = Str::random(6) . '-' . rand(1, 1000).'-'.uniqid();
        session()->put('unique_id', $unique_id);

        $country_list = $response['data'];

        $country = collect($country_list)->where('id', $customer_info['country_id'])->first()['iso3'];



        Stripe::setApiKey(env('STRIPE_SECRET'));
        header('Content-Type: application/json');

        $currency_code = 'USD';
        $currencies_not_supported_cents = ['BIF', 'CLP', 'DJF', 'GNF', 'JPY', 'KMF', 'KRW', 'MGA', 'PYG', 'RWF', 'UGX', 'VND', 'VUV', 'XAF', 'XOF', 'XPF'];

        $base_url = url('/');

        $check_customer = StripeCustomer::where('customer_id', $customer_info['id'])->first();
        if (!empty($check_customer)){
            $customer = \Stripe\Customer::update($check_customer['stripe_customer_id'], [
                'email' => $customer_info['email'],
                'address' => [
                    'city' => $customer_info['city'],
                    'country' => $country, // Pass the customer's country here
                    'line1' =>$customer_info['address_1'],
                    'line2' =>$customer_info['address_2'],
                    'postal_code' =>$customer_info['postal_code'],
                ],
            ]);

        }else{
            $customer = \Stripe\Customer::create([
                'email' => $customer_info['email'],
                'address' => [
                    'city' => $customer_info['city'],
                    'country' => $country, // Pass the customer's country here
                    'line1' =>$customer_info['address_1'],
                    'line2' =>$customer_info['address_2'],
                    'postal_code' =>$customer_info['postal_code'],
                ],
            ]);

            $stripe_customer = new StripeCustomer();
            $stripe_customer->customer_id = $customer_info['id'];
            $stripe_customer->stripe_customer_id = $customer->id;
            $stripe_customer->save();
        }

        $total_amt = ($product["price"] + $product["setup_fee"]) * 1;

        if ($product['billing_type'] == 0){
            $checkout_session = \Stripe\Checkout\Session::create([
                'payment_method_types' => ['card'],
                'line_items' => [[
                    'price_data' => [
                        'currency' => $currency_code,
                        'unit_amount' => in_array($currency_code, $currencies_not_supported_cents) ? (int)$total_amt : ($total_amt * 100),
                        'product_data' => [
                            'name' => 'alotel',
                            'images' => ['test.png'],
                        ],
                    ],
                    'quantity' => 1,
                ]],
                'customer' => $customer->id,
                'mode' => 'payment',
                'success_url' => $base_url.'/stripe-payment-success?session_id={CHECKOUT_SESSION_ID}&type='.$product['billing_type'],
                'cancel_url' => route('dashboard'),
                'metadata' => [
                    'order_id' => $unique_id,
                    'amount' => $total_amt,
                    'product_id' => $product_id,
                    'product_category' => $product['product_category'],
                    'billing_type' => $product['billing_type']
                ],
            ]);

            return response()->json(['id' => $checkout_session->id]);
        }else{

            $check = SubscriptionProduct::where('product_id', $product['id'])->count();
            if ($check == 0){
                $s_product = Product::create([
                    'name' => $product['name'],
                    'type' => 'service', // 'service' or 'good'
                    'description' => $product['name'],
                    // Additional data
                    'metadata' => [
                        'product_id' => $product['id'],
                        'custom_field' => $request->custom_field,
                    ],
                ]);

                $price = Price::create([
                    'unit_amount' => $product['price'] * 100,
                    'currency' => 'usd',
                    'recurring' => [
                        'interval' => 'month', // 'day', 'week', 'month', 'year'
                    ],
                    'product' => $s_product->id,
                ]);

                $subscription_product = new SubscriptionProduct();
                $subscription_product->product_id = $product['id'];
                $subscription_product->subs_product_id = $s_product->id;
                $subscription_product->subs_price_id = $price->id;
                $subscription_product->save();
            }else{
                $subscription_product = SubscriptionProduct::where('product_id', $product['id'])->first();
                try {
                    \Stripe\Product::retrieve($subscription_product['subs_product_id']);
                }catch (\Exception $e){
                    $s_product = Product::create([
                        'name' => $product['name'],
                        'type' => 'service', // 'service' or 'good'
                        'description' => $product['name'],
                        'metadata' => [
                            'product_id' => $product['id'],
                        ],
                    ]);

                    $price = Price::create([
                        'unit_amount' => $product['price'] * 100,
                        'currency' => 'usd',
                        'recurring' => [
                            'interval' => 'month', // 'day', 'week', 'month', 'year'
                        ],
                        'product' => $s_product->id,
                    ]);

                    $subscription_product = SubscriptionProduct::find($subscription_product->id);
                    $subscription_product->product_id = $product['id'];
                    $subscription_product->subs_product_id = $s_product->id;
                    $subscription_product->subs_price_id = $price->id;
                    $subscription_product->save();
                }

                $checkout_session = \Stripe\Checkout\Session::create([
                    'payment_method_types' => ['card'],
                    'line_items' => [[
                        'price' => $subscription_product->subs_price_id,
                        'quantity' => 1,
                    ]],
                    'customer' => $customer->id,
                    'mode' => 'subscription',
                    'success_url' => $base_url.'/stripe-payment-success?session_id={CHECKOUT_SESSION_ID}&type='.$product['billing_type'],
                    'cancel_url' => route('dashboard'),
                    'subscription_data' => [
                        'metadata' => [
                            'order_id' => $unique_id,
                            'amount' => $product['price'],
                            'product_id' => $product_id,
                            'product_category' => $product['product_category'],
                            'billing_type' => $product['billing_type'],
                            'customer_id' => $customer_info['id'],
                            'number' => $customer_info['number'],
                            'type' => $customer_info['type'],
                        ],
                    ],
                ]);
                return response()->json(['id' => $checkout_session->id]);
            }

        }

    }

    public function stripe_payment_success(Request $request)
    {
        fwrite($this->fp,"[stripe_payment_success] function called".PHP_EOL);
        Stripe::setApiKey(env('STRIPE_SECRET'));
        $session = \Stripe\Checkout\Session::retrieve($request->get('session_id'));

        if ($request->type == 0){

            if (session('unique_id') != $session->metadata->order_id){
                return redirect()->route('dashboard');
            }

            $amount = $session->metadata->amount;
            $product_id = $session->metadata->product_id;
            $product_category = $session->metadata->product_category;
            $billing_type = $session->metadata->billing_type;

            if ($billing_type != 0){
                notify()->error('Something went to wrong', 'Error');
                return redirect()->back();
            }

            $tran_id = $session->payment_intent;
            $check = CustomerTran::where('tran_id', $tran_id)->count();
            if ($check < 0){
                notify()->error('Something went to wrong',"Success");
                return redirect()->back();
            }

            $customer_info = Cookie::get('customer_info');
            $customer_info = json_decode($customer_info, true);


            $customer_tran = new CustomerTran();
            $customer_tran->order_id = $session->metadata->order_id;
            $customer_tran->customer_id = $customer_info['id'];
            $customer_tran->stripe_customer_id = $session->customer;
            $customer_tran->tran_id = $tran_id;
            $customer_tran->save();

            fwrite($this->fp,"[stripe_payment_success] Add balance api will be called ".PHP_EOL);
            //first add balance
            $response_data = CustomerLogic::add_balance($this->apiService, $customer_info, $amount);
            fwrite($this->fp,"Add balance api response ".json_encode($response_data).PHP_EOL);
            if ($response_data['response_code'] == 400){
                return redirect()->route('logout');
            }
            fwrite($this->fp,"User order api called ".json_encode($response_data).PHP_EOL);
            $response = $this->apiService->callAPI('POST', 'api/user_order/', [
                "id" => $customer_info['id'],
                "token" => Cookie::get('c_auth_token'),
                "action" =>  "order",
                "product_category" => $product_category,
                "product_id" => $product_id,
                "product_quantity" => 1,
                "device_id" => "",
                "payment_by" => 1,
                "callkit_token" =>  "BE6CAF775FFC2C1AAD28D9992E467156F044D68D21C59E4973C3A692DACAB03C",
                "apns_token"=>"63c7620a2c5ce0a1717850ecb559fb994c57bc6180f49c2e815efab09421f924",
                "username"=>$customer_info['number'],
                "password"=>$customer_info['dir_params']['password']
            ]);
            fwrite($this->fp,"[stripe_payment_success] User order api called ".json_encode($response_data).PHP_EOL);
            if ($response['response_code'] == 400){
                if ($response['error'] == 'Invalid key found.'){
                    return redirect()->route('logout');
                }
                notify()->error($response['error'],"Error");
                return redirect()->route('dashboard');
            }

            session()->forget('unique_id');
            notify()->success($response['success'],"Success");
            return redirect()->route('dashboard');
        }else{

            $subscription = \Stripe\Subscription::retrieve($session->subscription);
            $metadata = $subscription->metadata;

            if (session('unique_id') != $metadata->order_id){
                return redirect()->route('dashboard');
            }

            $amount = $metadata->amount;
            $product_id = $metadata->product_id;
            $product_category = $metadata->product_category;
            $billing_type = $metadata->billing_type;

            if ($billing_type == 0){
                notify()->error('Something went to wrong', 'Error');
            }

            $tran_id = $request->get('session_id');
            $check = CustomerTran::where('tran_id', $tran_id)->count();
            if ($check < 0){
                notify()->error('Something went to wrong',"Success");
                return redirect()->route('dashboard');
            }

            $customer_info = Cookie::get('customer_info');
            $customer_info = json_decode($customer_info, true);

            $customer_tran = new CustomerTran();
            $customer_tran->order_id = $metadata->order_id;
            $customer_tran->customer_id = $customer_info['id'];
            $customer_tran->stripe_customer_id = $session->customer;
            $customer_tran->tran_id = $tran_id;
            $customer_tran->save();


            //first add balance
            
            $response_data = CustomerLogic::add_balance($this->apiService, $customer_info, (int)$amount);
            if ($response_data['response_code'] == 400){
                return redirect()->route('logout');
            }
            fwrite($this->fp,"[stripe_payment_success] User order api called_1 ".PHP_EOL);
            $response = $this->apiService->callAPI('POST', 'api/user_order/', [
                "id" => $customer_info['id'],
                "token" => Cookie::get('c_auth_token'),
                "action" =>  "order",
                "product_category" => $product_category,
                "product_id" => $product_id,
                "product_quantity" => 1,
                "device_id" => "",
                "payment_by" => 1,
                "callkit_token" =>  "BE6CAF775FFC2C1AAD28D9992E467156F044D68D21C59E4973C3A692DACAB03C",
                "apns_token"=>"63c7620a2c5ce0a1717850ecb559fb994c57bc6180f49c2e815efab09421f924",
                "username"=>$customer_info['number'],
                "password"=>$customer_info['dir_params']['password']
            ]);
            fwrite($this->fp,"[stripe_payment_success] User order api called_1 response ".$response .PHP_EOL);
            if ($response['response_code'] == 400){
                if ($response['error'] == 'Invalid key found.'){
                    return redirect()->route('logout');
                }
                notify()->error($response['error'],"Error");
                return redirect()->route('dashboard');
            }

            session()->forget('unique_id');
            notify()->success('Subscription successfully added',"Success");
            return redirect()->route('dashboard');

        }

    }

    public function handleWebhook(Request $request)
    {
        fwrite($this->fp,"[handleWebhook] function called".PHP_EOL);
        $payload = $request->getContent();

        // Get the Stripe signature header
        $sigHeader = $request->header('Stripe-Signature');
    // Log the entire request information
        fwrite($this->fp,'[handleWebhook] Response '.$request);
            Stripe::setApiKey(env('STRIPE_SECRET'));

        try {
            $payload = $request->getContent();
            $sig_header = $request->header('Stripe-Signature');
            $event = Webhook::constructEvent($payload, $sig_header, 'whsec_JwFEa5emOYC5wPosysXXWESz8ae9ltYh');
        } catch (SignatureVerificationException $e) {
            return response()->json(['error' => 'Invalid signature'], 400);
        }

        fwrite($this->fp,"[handleWebhook] Event type".$event->type.PHP_EOL);
        // Handle the event
        switch ($event->type) {
            case 'invoice.payment_succeeded':
                    $invoice = $event->data->object;
                // Check if the payment is for a subscription renewal
                fwrite($this->fp,"[handleWebhook] ".$invoice->billing_reason ." Status".$invoice->status.PHP_EOL);
                if ($invoice->billing_reason == 'subscription_cycle' && $invoice->subscription && $invoice->status == 'paid') {
                        $subscriptionId = $invoice->subscription;
                        $subscription = StripeSubscription::retrieve($subscriptionId);
                        fwrite($this->fp,'[handleWebhook] Subscription data'.$subscription.PHP_EOL);
                    if ($subscription) {
                        $metadata = $subscription->metadata;
                        fwrite($this->fp,'[handleWebhook] Meta Data'.$metadata.PHP_EOL);
                        $amount = $metadata->amount;
                        $product_id = $metadata->product_id;
                        $product_category = $metadata->product_category;
                        $number = $metadata->number;
                        $customer_id = $metadata->customer_id;
                        fwrite($this->fp,"[handleWebhook] Number ".$number.PHP_EOL);
                        $response = CustomerLogic::get_customer_list($this->apiService, $number);
                        fwrite($this->fp,"[handleWebhook] Response ".json_encode($response).PHP_EOL);
                        if ($response['response_code'] != 200){
                            if ($response['error'] == 'Account Not Found'){
                                info('account not found');
                            }
                            info($response['error']);
                        }
                        if (!isset($response['data'][0]['accountid'])){
                            info('account not found');
                        }

                        $customer_info = $response['data'][0];
                        $customer_auth_token = $customer_info['token'];



                        //first add balance
                        $response_data = CustomerLogic::add_balance($this->apiService, $customer_info, (int)$amount);
                        fwrite($this->fp,"[handleWebhook] ".json_encode($response_data).PHP_EOL);
                        if ($response_data['response_code'] == 400){
                            info($response['error']);
                        }
                        fwrite($this->fp,"[handleWebhook] User_order api called".PHP_EOL);
                        $order_res = $this->apiService->callAPI('POST', 'api/user_order/', [
                            "id" => $customer_info['accountid'],
                            "token" => $customer_auth_token,
                            "action" =>  "order",
                            "product_category" => $product_category,
                            "product_id" => $product_id,
                            "product_quantity" => 1,
                            "device_id" => "",
                            "payment_by" => 1,
                            "callkit_token" =>  "BE6CAF775FFC2C1AAD28D9992E467156F044D68D21C59E4973C3A692DACAB03C",
                            "apns_token"=>"63c7620a2c5ce0a1717850ecb559fb994c57bc6180f49c2e815efab09421f924",
                            "username"=>$customer_info['number'],
                            "password"=>$customer_info['dir_params']['password']
                        ]);
                        fwrite($this->fp,"[handleWebhook] User_order api called".$order_res.PHP_EOL);
                        if ($order_res['response_code'] == 400){
                            if ($order_res['error'] == 'Invalid key found.'){
                                info('logout');
                            }
                            info($order_res['error']);
                        }else{
                            info('Renew success');
                        }

                    }
                }
                break;
            case 'charge.succeeded':
                $invoice = $event->data->object;
                if(
                    // @$invoice->payment_method_details &&
                    // @$invoice->payment_method_details->card &&
                    // @$invoice->payment_method_details->card->wallet &&
                    // @$invoice->payment_method_details->card->wallet->type &&
                    // @$invoice->payment_method_details->card->wallet->type == 'apple_pay'
                    $invoice->payment_intent
                ){

                    $paymentIntent = PaymentIntent::retrieve($invoice->payment_intent);
//                    $paymentIntent = PaymentIntent::retrieve('pi_3Pcxg12Nt0ZHEStj00YCmH4L');
                    if($paymentIntent){
                        Log::info("PaymentIntent ".json_encode($paymentIntent->metadata));
                        $metadata = $paymentIntent->metadata;
                        $amount_type = $metadata->amount_type;
                        Log::info("Amount Type ".$amount_type);
                        $amount = $invoice->amount/100;
                        if($amount_type == 'credit_amount'||$amount_type == 'product'){
                            $number = $metadata->number;
                            $customer_id = $metadata->customer_id;
                            $response = CustomerLogic::get_customer_list($this->apiService, $number);$customer_info['reseller_id'] = $response['data'][0]['reseller_id'];
                            $customer_info['accountid'] = $response['data'][0]['accountid'];
                            $customer_info['type'] = $metadata->type;
                            $request = CustomerLogic::add_balance($this->apiService, $customer_info, (int)$amount);
                            return response()->json(['message'=>$amount. '$  added to balance','response'=>$request], 200);
                        }
                        return response()->json(['success'], 200);

                    }
                }else{
                    return response()->json(['success'], 200);
                }
                // Check if the payment is for a subscription renewal
//                    $paymentIntent = PaymentIntent::retrieve($invoice->payment_method_details->card->wallet->type);
//                Log::info(json_encode($invoice->payment_method_details));
//                Log::info(json_encode($invoice->payment_method_details->card));
                break;
            // Add more cases for other event types as needed
            default:
                // Unexpected event type
                return response()->json(['error' => 'Unhandled event type'], 400);
        }

        return response()->json(['success' => true]);
    }

}