<?php

namespace App\Http\Controllers;
use \stdClass;
use App\Classes\Helpers;
use App\Helpers\Helper;
use App\Models\BodyType;
use App\Models\Ethnicity;
use App\Models\PrivateGalleryAccess;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Str;
use App\Http\Controllers\Controller;
use App\Models\User;
use App\Models\UserSeeking;
use App\Models\Relationship;
use App\Models\UserMeetup;
use App\Models\UserReport;
use App\Models\GroupMember;
use App\Models\Tap;
use App\Models\Group;
use App\Models\ReliabilityMember;
use App\Models\FavouriteUser;
use Carbon\Carbon;

class UserController extends Controller
{
    public function specific_user_details(Request $request)
    {
        $data = null;
        $current_date = Carbon::now()->format('Y-m-d');

        $response = Helper::success(200, 'success', $data);


        if(isset($request->otherUserId) && isset($request->userId) && isset($request->userLat) && isset($request->userLong)){
                    $other_user = User::where('users_id', $request->otherUserId)->with('other_user_stories', 'other_user_seekings', 'other_user_tribes')->first();
                    $other_user = $other_user->makeHidden(['password']);

                    //Concatenating Profile Picture with base URL
                    $other_user->profile_picture = config('base_urls.profile_images_base_url') . $other_user->profile_picture;
                    //Concatenating Story Pictures with base URL
                    foreach ($other_user->other_user_stories as $key => $story) {
                        $story->picture = config('base_urls.user_picture_base_url') . $story->picture;
                    }

                    $private_gallery_access = PrivateGalleryAccess::where([
                                                        ['requested_by_user_id', $request->userId],
                                                        ['requested_to_user_id', $request->otherUserId]
                                                    ])->first();

                    if ($private_gallery_access) {
                        //Requested Before either pending, Allowed, Denied
                        if ($private_gallery_access->status == "Allowed")
                            $other_user->other_private_gallery_access = "Allowed";
                        else if ($private_gallery_access->status == "Denied")
                            $other_user->other_private_gallery_access = "Denied";
                        else
                            $other_user->other_private_gallery_access = "Pending";
                    } else {
                        // Case where request is allowed to sent -- First time request
                        $other_user->other_private_gallery_access = "Request Allowed";
                    }

                    //Human Friendly date of birth
                    $other_user->age = Carbon::parse($other_user->birthday)->age;

                    //Measure distance in kilometers, Pass emplty string in last parameter to get distance in Miles
                    $other_user->distance_away = Helper::calculate_miles($request->userLat, $request->userLong, $other_user->user_lat, $other_user->user_long, 'K');

                    //Ethnicity
                    $ethnicity = Ethnicity::where([
                                            ['ethnicity_id', $other_user->ethnicity_id],
                                            ['status', 'Active']
                                        ])->first();

                    if ($ethnicity)
                        $other_user->ethnicity = $ethnicity->ethnicity;
                    else
                        $other_user->ethnicity = null;

                    //Body Type
                    $body_type = BodyType::where([
                                                ['body_type_id', $other_user->body_type_id],
                                                ['status', 'Active']
                                            ])->first();

                    if ($body_type)
                        $other_user->body_type = $body_type->body_type;
                    else
                        $other_user->body_type = null;

                    //Relation
                    $relationship = Relationship::where([
                                                ['relationship_id', $other_user->relationship_id],
                                                ['status', 'Active']
                                            ])->first();
                    if ($relationship)
                        $other_user->relationship_status = $relationship->relationship;
                    else
                        $other_user->relationship_status = null;

                     //Tapped Flag
                    $isTapped =Tap::where([ ['tapping_to_user_id', $request->otherUserId], ['tapping_by_user_id', $request->userId] ])->count();
                    if ($isTapped > 0)
                        $other_user->is_tapped = true;
                    else
                        $other_user->is_tapped = false;

                    //is Meeup Flag
                    $isMeetup = UserMeetup::where([ ['met_to_user_id', $request->otherUserId], ['met_by_user_id', $request->userId] ])->count();
                    if ($isMeetup > 0)
                        $other_user->is_meetup = true;
                    else
                        $other_user->is_meetup = false;

                    //for online status
                    $userData = User::where('users_id', $request->otherUserId)->first();
                    if($userData->online == 'Yes')
                        $other_user->is_online = true;
                    else
                        $other_user->is_online = false;





            //is Favourite Flag
            $isMeetup = FavouriteUser::where([ ['favourite_user_id', $request->otherUserId], ['users_id', $request->userId] ])->count();
            if ($isMeetup > 0)
                $other_user->is_favourite = true;
            else
                $other_user->is_favourite = false;








                    //Calculating Reliability
                    $group = Group::where('users_id', $request->otherUserId)->whereDate('ending_date', '<', $current_date)->orderBy('ending_date', 'DESC')->first();

                    if($group) {

                        $group_members = GroupMember::where('group_id', $group->group_id)->count();

                        if ($group_members > 0) {
                            $reliability_members = ReliabilityMember::where('group_id', $group->group_id)->count();
                            $reliability_count = ($reliability_members/$group_members) * 100;
//                            $other_user->reliability = 12;
                            $other_user->reliability = (int) $reliability_count;
                        } else {
                            $other_user->reliability = 0;
                        }
                    } else {
                        $other_user->reliability = 0;
                    }




                           
                    //GROUPS ATTENDED
                    $attendedGroups = array();
                    $attendedGroupsQuery = GroupMember::where([ ['users_id', $request->otherUserId], ['status', 'Accepted'] ])->get();
                        if(sizeof($attendedGroupsQuery) > 0){
                            foreach($attendedGroupsQuery as $k){
                                $attendedGroupDetail= Group::select('group_id', 'users_id', 'status', 'group_category_id', 'cover_photo')
                                                                                    ->with('category')
                                                                                    ->where('group_id', $k->group_id)->first();

                                if($attendedGroupDetail->cover_photo != null && $attendedGroupDetail->cover_photo != "")
                                    $attendedGroupDetail->cover_photo = config('base_urls.group_cover_photo_base_url').$attendedGroupDetail->cover_photo;
                                else
                                    $attendedGroupDetail->cover_photo = "";

                                array_push($attendedGroups, $attendedGroupDetail);
                            }
                                  $other_user->attended_groups = $attendedGroups;

                            }else{
                                  $other_user->attended_groups = [];
                            }



                    
                        //GROUPS HOSTED
                    $hostedGroupsData= Group::select('group_id', 'users_id', 'status', 'group_category_id', 'cover_photo')
                                                    ->with('category')
                                                    ->where([ ['users_id', $request->otherUserId], ['status', 'Active'] ])->get();

                    if(sizeof($hostedGroupsData) > 0){
                            foreach($hostedGroupsData as $k){
                                if($k->cover_photo != null && $k->cover_photo != "")
                                    $k->cover_photo = config('base_urls.group_cover_photo_base_url').$k->cover_photo;
                                else
                                    $k->cover_photo = "";
                            }
                        $other_user->hosted_groups = $hostedGroupsData;

                    }else{
                        $other_user->hosted_groups = [];
                    }




                    $response = Helper::success(200, 'success', $other_user);
        }else{
            $response = Helper::failed(404, 'error', 'All fields are needed');

        }

        return Helper::custom_response($response);
    }


    
    public function tap_user(Request $request)
    {
        
        if(isset($request->tappingToUserId) && isset($request->tappingByUserId) ){
                $checkAlreadyTapped = Tap::where([ ['tapping_to_user_id', $request->tappingToUserId], ['tapping_by_user_id', $request->tappingByUserId] ])->count();
                    if($checkAlreadyTapped == 0){
                            Tap::create([
                                'tapping_to_user_id' => $request->tappingToUserId,
                                'tapping_by_user_id'=>  $request->tappingByUserId,
                            ]);
                          

                        $response = Helper::success(200, 'success', 'Tapped Successfully');

                    }else{
                        $response = Helper::failed(404, 'error', 'Already Tapped');
                    }
        }else{
            $response = Helper::failed(404, 'error', 'All fields are needed');
        }

        return Helper::custom_response($response);
    }

    public function meet_user(Request $request)
    {
       
        if(isset($request->metToUserId) && isset($request->metByUserId) ){

            //Is Verified User
            $userMeetups = UserMeetup::where('met_to_user_id', $request->metToUserId)->count();


            if ($userMeetups >= 10) {
                $user = User::find($request->metToUserId);
                $user->verified = 'Yes';
                $user->save();
            }

                $checkAlreadyMet = UserMeetup::where([ ['met_to_user_id', $request->metToUserId], ['met_by_user_id', $request->metByUserId] ])->count();
                    if($checkAlreadyMet == 0){
                        UserMeetup::create([
                                'met_to_user_id' => $request->metToUserId,
                                'met_by_user_id'=>  $request->metByUserId,
                            ]);
                          

                        $response = Helper::success(200, 'success', 'Meetup Successfully');
                    }else
                        $response = Helper::failed(404, 'error', 'Already Met');
        }else
            $response = Helper::failed(404, 'error', 'All params required');

        return Helper::custom_response($response);
    }

    public function report_user(Request $request)
    {
        if(isset($request->reportedToUserId) && isset($request->reportedByUserId) && isset($request->reason) ){
                $checkAlreadyReported = UserReport::where([ ['reported_to_user_id', $request->reportedToUserId], ['reported_by_user_id', $request->reportedByUserId] ])->count();
                    if($checkAlreadyReported == 0){
                        UserReport::create([
                                        'reported_to_user_id' => $request->reportedToUserId,
                                        'reported_by_user_id'=>  $request->reportedByUserId,
                                        'reason'=>  $request->reason,
                                        'status' => 'Reported'
                                    ]);

                        $response = Helper::success(200, 'success', 'Reported Successfully');

                    }else{
                        $response = Helper::failed(404, 'error', 'Already Reported');
                    }
        }else{
            $response = Helper::failed(404, 'error', 'All fields are needed');
        }

        return Helper::custom_response($response);
    }


    

    public function get_discovered_user_groups(Request $req){
        $response = array();


        if (isset($req->usersId) && isset($req->latitude) && isset($req->longitude) ){
            $mainDataset = array();

            $nearbyGroups = Group::with('category','tribes', 'features','rules','members')->where('status', 'Active')->get();
            $nearbyUsers = User::where('status', 'Active')->where('users_id','!=',$req->usersId)->get();

            foreach($nearbyGroups as $k){
                $customObject = new stdClass();
                $customObject->group_id = $k->group_id;
                $customObject->users_id = null;
                $customObject->name = $k->category->category;

                $distanceInKm = Helper::calculate_miles($req->latitude, $req->longitude, $k->group_lat, $k->group_long, 'K');
                $customObject->distance_away = $distanceInKm;
                $customObject->latitude = $k->group_lat;
                $customObject->longitide = $k->group_long;

                $timeAgo = Helper::time_elapsed_string_group($k->created_at);
                $customObject->time_ago = $timeAgo;

                if($k->cover_photo != null && $k->cover_photo != "")
                    $customObject->picture = config('base_urls.group_cover_photo_base_url').$k->cover_photo;
                else
                    $customObject->picture = "";


                if($k->verified == 'Yes')
                    $customObject->is_verified = True;
                else
                    $customObject->is_verified = False;

                $customObject->total_active_members = GroupMember::where([ ['group_id', $k->group_id], ['status', 'Accepted'] ])->count();
                $customObject->is_online = null;
                $customObject->object_type = 'Group';

                if($distanceInKm <= 200)
                    array_push($mainDataset, $customObject);
            }


            foreach($nearbyUsers as $k){
                $customObject = new stdClass();
                $customObject->users_id = $k->users_id;
                $customObject->group_id = null;
                $customObject->name = $k->user_name;

                $distanceInKm = Helper::calculate_miles($req->latitude, $req->longitude, $k->user_lat, $k->user_long, 'K');
                $customObject->distance_away = $distanceInKm;
                $customObject->latitude = $k->user_lat;
                $customObject->longitide = $k->user_long;

                $customObject->time_ago = null;

                //for online status
                $userData = User::where('users_id', $k->users_id)->first();
                if($userData->online == 'Yes')
                    $customObject->is_online = true;
                else
                    $customObject->is_online = false;

                if($k->profile_picture != null && $k->profile_picture != "")
                    $customObject->picture = config('base_urls.profile_images_base_url').$k->profile_picture;
                else
                    $customObject->picture = "";


                if($k->verified == 'Yes')
                    $customObject->is_verified = True;
                else
                    $customObject->is_verified = False;

                $customObject->total_active_members = null;
                $customObject->object_type = 'User';

                if($distanceInKm <= 200)
                       array_push($mainDataset, $customObject);

            }


            $mainDataset = array_values(collect($mainDataset)->sortBy('distance_away')->toArray());


            $response["code"] = 200;
            $response["status"] = "success";
            $response["data"] = $mainDataset;

        } else {
            $response["code"] = 404;
            $response["status"] = "error";
            $response["message"] = "All fields are needed.";
        }

        return response()
            ->json(array(
                'status' => $response["status"],
                isset($response["message"]) ? 'message' : 'data' => isset($response["message"]) ? $response["message"] : $response["data"]))
            ->header('Content-Type', 'application/json');

    }


    public function favourite_user(Request $request)
    {
       
        if(isset($request->userId) && isset($request->favouriteUserId) ){

            //Is already fav
            $alreadyFav = FavouriteUser::where([ ['users_id', $request->userId], ['favourite_user_id', $request->favouriteUserId] ])->count();
            
            if ($alreadyFav == 0) {
                FavouriteUser::insert([
                                    'users_id' => $request->userId,
                                    'favourite_user_id' => $request->favouriteUserId,
                                ]);
                $response = Helper::success(200, 'success', 'Added to favourites');
           
            }else{
                FavouriteUser::where([ ['users_id', $request->userId], ['favourite_user_id', $request->favouriteUserId] ])->delete();
                $response = Helper::success(200, 'success', 'Removed from favourites');

            }


        }else{
            $response = Helper::failed(404, 'error', 'All fields are needed');
        }

        return Helper::custom_response($response);
    }


    public function online_user(Request $request)
    {
       
        if(isset($request->userId)){

            //Is already fav
             User::where('users_id', $request->userId)->update(['online' => 'Yes']);

                $response = Helper::success(200, 'success', 'User is online');

        }else{
            $response = Helper::failed(404, 'error', 'All fields are needed');
        }

        return Helper::custom_response($response);
    }





public function offline_user(Request $request)
{
   
    if(isset($request->userId)){
        //Is already fav
         User::where('users_id', $request->userId)->update(['online' => 'No']);

            $response = Helper::success(200, 'success', 'User is offline');

    }else{
        $response = Helper::failed(404, 'error', 'All fields are needed');
    }

    return Helper::custom_response($response);
}




public function users_groups_count_in_radius(Request $req){
    $response = array();


    if (isset($req->latitude) && isset($req->longitude) ){
        $mainDataset = array();
        $nearbyGroups = array();
        $nearbyUsers = array();

        $allGroups = Group::with('category','tribes', 'features','rules','members')->where('status', 'Active')->get();
        $allUsers = User::where('status', 'Active')->get();

        foreach($allGroups as $k){
            $distanceInKm = Helper::calculate_miles($req->latitude, $req->longitude, $k->group_lat, $k->group_long, 'K');
            if($distanceInKm <= 200)
                array_push($nearbyGroups, $k);
        }


        foreach($allUsers as $k){
            $distanceInKm = Helper::calculate_miles($req->latitude, $req->longitude, $k->user_lat, $k->user_long, 'K');
            if($distanceInKm <= 200)
                   array_push($nearbyUsers, $k);
        }


        $mainDataset = new stdClass();
        $mainDataset->users_count = sizeof($nearbyUsers);
        $mainDataset->groups_count = sizeof($nearbyGroups);

        $response["code"] = 200;
        $response["status"] = "success";
        $response["data"] = $mainDataset;
 
    } else {
        $response["code"] = 404;
        $response["status"] = "error";
        $response["message"] = "All fields are needed.";
    }

    return response()
        ->json(array(
            'status' => $response["status"],
            isset($response["message"]) ? 'message' : 'data' => isset($response["message"]) ? $response["message"] : $response["data"]))
        ->header('Content-Type', 'application/json');

}


}
